@@ -1,19 +1,20 @@
-;;; $Id: dtk-tcl.el,v 5.0 1996/11/22 18:04:26 raman Exp $
+;;; $Id: dtk-tcl.el,v 7.0 1997/11/13 15:32:30 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Interfacing to the Dectalk via TCL. 
 ;;; Keywords: Dectalk, TCL
 ;;{{{  LCD Archive entry: 
+
 ;;; LCD Archive Entry:
-;;; emacspeak| T. V. Raman |raman@crl.dec.com 
+;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $date: $ |
-;;;  $Revision: 5.0 $ | 
+;;; $Date: 1997/11/13 15:32:30 $ |
+;;;  $Revision: 7.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-
+;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -39,6 +40,7 @@
 (require 'cl)
 (declaim  (optimize  (safety 0) (speed 3)))
 (require 'dtk-macros)
+(require 'dtk-voices)
 (require 'emacspeak-sounds)
 (require 'emacspeak-pronounce)
 (eval-when (compile)
@@ -89,8 +91,14 @@ dtk-set-character-scale bound to \\[dtk-set-character-scale]. ")
 
 (defvar dtk-capitalize nil
   "Non-nil means produce a beep to indicate  capitalization.
- Do not set this variable by hand, use command dtk-capitalize
-bound to \\[dtk-capitalize]. ")
+ Do not set this variable by hand, use command dtk-toggle-capitalization
+bound to \\[dtk-toggle-capitalization]. ")
+
+(defvar dtk-allcaps-beep nil
+  "Non-nil means produce a beep to indicate upper case words in conjunction with
+split caps Do not set this variable by hand, use command
+dtk-toggle-allcaps-beep bound to \\[dtk-toggle-allcaps-beep]. ")
+
 
 (defconst dtk-punctuation-mode-alist
   '(("some" . "some" )
@@ -112,6 +120,8 @@ bound to \\[dtk-capitalize]. ")
   "Rate at which dectalk talks.
 Do not modify this variable directly; use command  dtk-set-rate
  bound to \\[dtk-set-rate]. ")
+;;;declared here to help compilation
+(defvar voice-lock-mode nil)
 
 ;;}}}
 ;;{{{  Helpers to handle invisible text:
@@ -126,19 +136,18 @@ Do not modify this variable directly; use command  dtk-set-rate
      (defsubst skip-invisible-forward  ()
      (and (text-invisible-p (point))
      (goto-char
-      (or (next-single-property-change (point) 'invisible )
-      (point-max)))))
+       (next-single-property-change (point) 'invisible 
+      (current-buffer) (point-max)))))
 ;;; Delete invisible text from current buffer:
 (defsubst skip-invisible-backward  ()
      (and (text-invisible-p (point))
      (goto-char
-      (or (previous-single-property-change (point) 'invisible )
-      (point-min)))))
+       (previous-single-property-change (point) 'invisible 
+      (current-buffer) (point-min)))))
 
 (defsubst delete-invisible-text ()
   (goto-char (point-min))
-  (let ((start  (point ))
-        (end nil ))
+  (let ((start  (point )))
     (while (not (eobp))
       (cond
        ((text-invisible-p (point ))
@@ -153,6 +162,28 @@ Do not modify this variable directly; use command  dtk-set-rate
 ;;}}}
 ;;{{{  define inline functions first
 
+(defsubst dtk-silence (duration &optional force)
+  "Produce silence"
+  (declare (special dtk-quiet dtk-speaker-process
+                    dtk-speak-driver-initialized))
+  (unless dtk-quiet 
+    (when dtk-speak-driver-initialized
+      (process-send-string dtk-speaker-process
+                           (format "sh %d%s\n"
+                                   duration
+                                   (if force ";d" ""))))))
+
+(defsubst dtk-tone (pitch duration &optional force)
+  "Produce a tone."
+  (declare (special dtk-quiet dtk-speaker-process
+                    dtk-speak-driver-initialized))
+  (unless dtk-quiet 
+    (when dtk-speak-driver-initialized
+         (process-send-string dtk-speaker-process
+                         (format "t %d %d%s\n"
+pitch duration
+(if force ";d" ""))))))
+
 ;;; helper function:
 
 ;;; Quote the string in current buffer so tcl does not barf.
@@ -198,21 +229,40 @@ Do not modify this variable directly; use command  dtk-set-rate
     (while (re-search-forward " [A-Z] " nil t)
     (replace-match  " cap \\&" ))))
 
+(defvar dtk-speak-nonprinting-chars t
+  "*Non nil value means non printing characters  should be
+spoken as their octal value.
+Set this to t to avoid a dectalk bug that makes the speech box die if
+it seems some accented characters in certain contexts.")
+
+(make-variable-buffer-local 'dtk-speak-nonprinting-chars)
+
 (defsubst dtk-fix-control-chars ()
-  (declare (special dtk-character-to-speech-table))
-  (let ((control "[\001-\037]"))
+  (declare (special dtk-character-to-speech-table
+                    dtk-speak-nonprinting-chars))
+  (let ((specials "[\000-\037\177-\377]")
+        (char nil))
     (goto-char (point-min ))
-    (save-match-data 
-      (while (re-search-forward control nil t )
-        (or  (= (char-after (match-beginning 0)) 10)
-             (= (char-after (match-beginning 0)) 9)
-             (replace-match
-              (format " %s " (aref  dtk-character-to-speech-table
-                                    (char-after (match-beginning 0 ))))))))))
+    (when  dtk-speak-nonprinting-chars
+      (save-match-data 
+        (while (re-search-forward specials nil t )
+          (setq char (char-after (match-beginning 0)))
+          (or
+           (= char  10)                 ;do nothing 
+           (= char  9)                  ; do nothing 
+           (replace-match
+            (format " %s " (aref  dtk-character-to-speech-table char)))))))))
+
+
+;;; replace ".," with ". ," so we hear it.
+
+(defsubst dtk-fix-dot-comma ()
+  (goto-char (point-min))
+  (while (search-forward ".," nil t)
+    (replace-match ". ,")))
 
 (defsubst  dtk-quote(mode ) 
-  (declare (special dtk-cleanup-patterns
-                    dtk-capitalize))
+  (declare (special dtk-cleanup-patterns))
   (goto-char (point-min))
       ;;; First cleanup  repeated patterns:
   (save-excursion
@@ -225,6 +275,28 @@ Do not modify this variable directly; use command  dtk-set-rate
   ;;; fix control chars
   (dtk-fix-control-chars))
 
+
+(defsubst dtk-fix-backslash ()
+    (goto-char (point-min))
+    (while (search-forward "\\" nil t)
+    (replace-match " backslash ")))
+
+;;; efficient quoting function for use in dtk-say
+(defsubst  dtk-quick-quote(string ) 
+  (let ((dtk-scratch-buffer (get-buffer-create " *dtk-scratch-buffer* "))
+                            (inhibit-read-only t))
+        (save-excursion
+          (set-buffer dtk-scratch-buffer))
+        (erase-buffer)
+        (insert string)
+        (goto-char (point-min))
+    ;;; dtk will think it's processing a command otherwise:
+  (dtk-fix-brackets "all")
+  (dtk-fix-backslash)
+  ;;; fix control chars
+  (dtk-fix-control-chars)))
+
+
 ;;; Moving  across a chunk of text. 
 ;;; A chunk  is specified by a punctuation followed by whitespace
 ;;; or  multiple blank lines
@@ -232,7 +304,7 @@ Do not modify this variable directly; use command  dtk-set-rate
 ;;; or a parenthesis grouping start or end 
 ;;; leaves point at the end of the chunk.
 ;;; returns  distance moved; nil if stationery
-(defvar dtk-chunk-separator-syntax ".<>()$\"\'"
+(defvar dtk-chunk-separator-syntax ".>)$\""
   "Syntax string to identify chunks when splitting text. ")
 ; make it buffer local:
 (make-variable-buffer-local 'dtk-chunk-separator-syntax)
@@ -253,91 +325,94 @@ Do not modify this variable directly; use command  dtk-set-rate
                                (= x 32 )))
                    dtk-chunk-separator-syntax)))
 
-;;; Invariance: at front of visible text:
-;;; stop moving at clause boundary 
+
+;;; invariance: looking at complement
+;;; move across the complement and the following separator
+;;; return value is a boolean indicating if we moved.
+;;; side-effect is to move across a chunk
 (defsubst  dtk-move-across-a-chunk (separator complement )
-  (skip-syntax-forward separator)
-  (let ((done nil )
-        (orig (point ))
-        (moved (skip-syntax-forward complement)))
-    (while (and  (not done )
-                 (not (eobp )))
-      (skip-syntax-forward separator)
-      (cond
-                                        ; fail if you did not move 
-       ((zerop moved) (goto-char orig )
-        (setq done t))
-       ;; Success if we found non-punctuation delimiter
-       ((not (= (char-syntax (preceding-char ))
-                46;; "."
-                ))
-        (setq done t ))
-                                        ; if  sep is punct ,   skip   ws and succeed 
-       ((or (eolp)
-            (and (char-after  (point))
-                 (= 32 (char-syntax (preceding-char )))))
-        (skip-syntax-forward " ")
-        (setq done t))
-                                        ; otherwise continue looking 
-       (t (setq moved
-                (skip-syntax-forward complement)))))
-    (or (not (zerop moved))
-        (re-search-forward "\n\n+" nil t ))))
+  (> (+ (skip-syntax-forward complement)
+      (skip-syntax-forward separator))
+   0))
+      
+;;; efficient way of voice changing
+
+(defsubst dtk-speak-using-voice (voice text)
+  (declare (special dtk-voice-reset-code))
+  (unless (eq 'inaudible voice )
+            (process-send-string dtk-speaker-process
+             (format "q {%s%s%s }\n"
+                     (dtk-get-voice-command voice )
+                     text
+                     dtk-voice-reset-code))))
 
                                         ;Internal function used by dtk-speak to send text out.
                                         ;Handles voice locking etc.
                                         ; assumes in dtk-scratch-buffer
-                                        ;start and end give the extent of the text to be spoken. .
+                                        ;start and end give the extent of the
+;text to be spoken. 
+; note that property auditory-icon at the start  of a clause
+; causes the sound to be queued.
 (defsubst dtk-format-text-and-speak (start end )
-  (declare (special voice-lock-mode  dtk-current-voice
-                    dtk-speaker-process ))
+  (declare (special voice-lock-mode dtk-speaker-process
+                    emacspeak-use-auditory-icons))
+  (when (and emacspeak-use-auditory-icons
+             (get-text-property start 'auditory-icon))
+    (emacspeak-queue-auditory-icon (get-text-property start 'auditory-icon)))
   (cond
-   ((and (boundp 'voice-lock-mode) voice-lock-mode)
+   (voice-lock-mode
     (let ((last  nil)
           (personality (get-text-property start 'personality )))
       (while (and (< start end )
                   (setq last
-                         (next-single-property-change  start 'personality
-                                                          (current-buffer) end)))
+                        (next-single-property-change  start 'personality
+                                                      (current-buffer) end)))
         (if personality 
-            (dtk-with-voice
-             personality 
-             (process-send-string dtk-speaker-process
-                                  (format "queue_speech { %s }  \n"
-                                          (buffer-substring start last ))))
+            (dtk-speak-using-voice personality 
+                                   (buffer-substring start last ))
           (process-send-string dtk-speaker-process
-                               (format "queue_speech { %s }  \n"
+                               (format "q {%s }\n"
                                        (buffer-substring  start last))))
         (setq start  last
               personality (get-text-property last  'personality))) ; end while
       ))                                ; end clause 
    (t (process-send-string dtk-speaker-process
-                           (format "queue_speech  { %s }  \n"
+                           (format "q {%s }\n"
                                    (buffer-substring start end  ))))))
 
                                         ;Force the speech. 
 (defsubst dtk-force ()
-  (declare (special dtk-speaker-process ))
-  (process-send-string dtk-speaker-process
-                       "dectalk_speak {} \n"))
+  (declare (special dtk-speaker-process))
+       (process-send-string dtk-speaker-process
+                            "d\n"))
 
                                         ;Write out the string to the dectalk via TCLSH.
                                         ;No quoting is done, if want to quote the text, see dtk-speak
 (defsubst dtk-dispatch (string)
   (declare (special dtk-speaker-process
+                    dtk-speak-driver-initialized
                     dtk-quiet)) 
   (unless dtk-quiet
-          (process-send-string dtk-speaker-process
-                               (format  "dectalk_say { %s }\n " string ))))
+    (when dtk-speak-driver-initialized
+         (process-send-string dtk-speaker-process
+                              (format  "dectalk_say { %s}\n" string )))))
 
 (defsubst dtk-stop ()
   "Stop speech now. "
   (interactive)
-  (declare (special dtk-speaker-process ))
-  (process-send-string dtk-speaker-process
-                       (format "dectalk_stop \n" )))
+  (declare (special dtk-speaker-process
+                    dtk-speak-driver-initialized))
+  (when dtk-speak-driver-initialized
+       (process-send-string dtk-speaker-process
+                            (format "s\n" ))))
+
+
+(defsubst dtk-reset-default-voice()
+  (declare (special dtk-default-voice ))
+  (dtk-dispatch (dtk-get-voice-command dtk-default-voice )))
 
 ;;}}}
+
 ;;{{{  adding cleanup patterns:
 
 (defun dtk-add-cleanup-pattern (&optional delete ) 
@@ -391,12 +466,19 @@ will say ``aw fifteen dot'' when speaking the string
                        (/ (- (match-end 0 ) (match-beginning 0))
                           (length string))
                        (if (string= " " pattern)
-                           " space " pattern) )
+                           " space " string) )
              "")))))))
 
 ;;; Uses the syntax table belonging to the buffer that owns the text
 ;;; to parse and speak the text intelligently.
 
+(defvar dtk-speak-block-and-synchronize nil
+  "If t, then dtk-speak behaves  like when called
+with wait set to t")
+
+(defvar dtk-speak-treat-embedded-punctuations-specially t
+  "*If T then speech not split at embedded punctuations.")
+
 
 (defun dtk-speak (text &optional wait)
   "Speak the text string on the  dectalk, 
@@ -407,161 +489,120 @@ has been completed.
 If outline-minor-mode is on and selective display is in effect,
 only speak upto the first ctrl-m. "
   (declare (special dtk-speaker-process dtk-stop-immediately
+                    dtk-speak-nonprinting-chars
+                    dtk-speak-treat-embedded-punctuations-specially
                     dtk-quiet  dtk-chunk-separator-syntax
-                    voice-lock-mode dtk-current-voice  dtk-punctuation-mode
-                    emacspeak-pronounce-pronunciation-table selective-display ))
-                                        ; If you dont want me to talk, I wont.
+                    voice-lock-mode   dtk-punctuation-mode
+                    emacspeak-pronounce-pronunciation-table
+                    selective-display ))
+                                        ; If you dont want me to talk,
+                                        ; I wont.
   (unless  dtk-quiet
                                         ; ensure  the process  is live
     (or (eq 'run (process-status dtk-speaker-process ))
         (dtk-initialize) )
                                         ; flush previous speech if asked to 
-    (when dtk-stop-immediately
-      (dtk-stop )
-      (if (and (boundp 'voice-lock-mode ) voice-lock-mode)
-          (dtk-dispatch
-           (dtk-get-voice-command dtk-current-voice ))))
+    (when dtk-stop-immediately (dtk-stop ))
     (or (stringp text) (setq text (format "%s" text )))
     (let ((ctrl-m nil ))
-      (when (and (boundp  'selective-display )
-                 selective-display 
-                 (setq ctrl-m (string-match "\015" text )))
-                 (emacspeak-auditory-icon 'ellipses)
-        (setq text (substring  text 0 ctrl-m ))))
+      (and selective-display 
+           (setq ctrl-m (string-match "\015" text ))
+           (setq text (substring  text 0 ctrl-m ))
+           (emacspeak-auditory-icon 'ellipses)))
     (let ((syntax-table (syntax-table ))
-          (pronunciation-table (and (boundp 'emacspeak-pronounce-pronunciation-table)
-                                    emacspeak-pronounce-pronunciation-table))
+          (pronunciation-table
+           emacspeak-pronounce-pronunciation-table)
+          (use-auditory-icons emacspeak-use-auditory-icons)
           (inherit-chunk-separator-syntax dtk-chunk-separator-syntax )
+          (inherit-speak-nonprinting-chars dtk-speak-nonprinting-chars)
           (complement-separator(dtk-complement-chunk-separator-syntax ))
+          (dtk-scratch-buffer (get-buffer-create " *dtk-scratch-buffer* "))
           (start 1)
           (end nil )
-          (dtk-scratch-buffer (get-buffer-create " *dtk-scratch-buffer* "))
           (mode dtk-punctuation-mode)
-          (voice-lock (and (boundp 'voice-lock-mode)
-                           voice-lock-mode )))
+          (voice-lock voice-lock-mode ))
       (save-excursion
         (set-buffer dtk-scratch-buffer )
         (let ((inhibit-read-only t))
           (erase-buffer)
                                         ; inherit environment
-          (setq dtk-chunk-separator-syntax inherit-chunk-separator-syntax)
+          (setq dtk-chunk-separator-syntax
+                inherit-chunk-separator-syntax
+                emacspeak-use-auditory-icons use-auditory-icons
+                dtk-speak-nonprinting-chars inherit-speak-nonprinting-chars)
           (set-syntax-table syntax-table )
           (setq voice-lock-mode voice-lock)
-          (insert " ")
           (insert  text)
           (delete-invisible-text)
           (when pronunciation-table
             (emacspeak-pronounce-apply-pronunciations pronunciation-table))
           (dtk-quote mode))
         (goto-char (point-min))
+        (skip-syntax-forward inherit-chunk-separator-syntax)
         (while (and (not (eobp))
                     (dtk-move-across-a-chunk
-                     inherit-chunk-separator-syntax complement-separator))
-          (setq end (point ))
-          (dtk-format-text-and-speak  start end )
-          (setq start  end))            ; end while
+                     inherit-chunk-separator-syntax
+                     complement-separator))
+                                        ;if we matched a punctuation,
+                                        ;treat this as a chunk only if the punctuation is followed
+                                        ;by white space
+          (unless (and dtk-speak-treat-embedded-punctuations-specially
+                   (char-after  (point)) 
+                       (= (char-syntax (preceding-char )) ?.)
+                       (not (= 32 (char-syntax (following-char )))))
+            (setq end (point ))
+            (dtk-format-text-and-speak  start end )
+            (setq start  end)))         ; end while
                                         ; process trailing text
-        (or (eobp) 
-            (dtk-format-text-and-speak start (point-max )))))
-    (when  wait
-      (accept-process-output dtk-speaker-process 0 1))
+        (or  (= start (point-max))
+             (dtk-format-text-and-speak start (point-max)))))
+    (when wait (accept-process-output dtk-speaker-process ))
     (dtk-force)
-    (when wait 
-      (accept-process-output dtk-speaker-process ))))
+    (when wait (accept-process-output dtk-speaker-process ))))
 
 (defun dtk-speak-list (text )
   "Speak a  list of strings. "
-  (declare (special dtk-speaker-process dtk-stop-immediately
-                    voice-lock-mode dtk-punctuation-mode))
-                                        ; ensure  the process  is live 
-  (unless (eq 'run (process-status dtk-speaker-process ))
-    (dtk-initialize) )
-                                        ; flush previous speech if asked to 
-  (when dtk-stop-immediately (dtk-stop ))
-  (let ((syntax-table (syntax-table ))
-        (start 1)
-        (end nil )
-        (inherit-chunk-separator-syntax dtk-chunk-separator-syntax )
-        (complement-separator(dtk-complement-chunk-separator-syntax ))
-        (dtk-scratch-buffer (get-buffer-create " *dtk-scratch-buffer* "))
-        (mode dtk-punctuation-mode)
-        (voice-lock voice-lock-mode ))
-    (save-excursion
-      (set-buffer dtk-scratch-buffer )
-      (set-syntax-table syntax-table )
-      (setq dtk-chunk-separator-syntax inherit-chunk-separator-syntax)
-      (setq voice-lock-mode voice-lock)
-      (while text 
-        (let ((inhibit-read-only t))
-        (erase-buffer)
-        (insert  (car text))
-        (setq text (cdr text))
-        (dtk-quote mode)
-        (goto-char (point-min)))
-        (while (and (not (eobp))
-                    (dtk-move-across-a-chunk
-                     inherit-chunk-separator-syntax complement-separator))
-          (setq end (point ))
-          (dtk-format-text-and-speak  start end )
-          (setq start  end))            ; end while
-                                        ; process trailing text 
-        (unless (eobp) 
-          (dtk-format-text-and-speak start (point-max )))
-        (dtk-tone 500 50)
-        (setq start 1 end nil )
-        )))
-  (dtk-force))
+  (declare (special dtk-speaker-process dtk-stop-immediately))
+  (let ((dtk-scratch-buffer (get-buffer-create " *dtk-scratch-buffer* "))
+        (contents nil)
+        (inhibit-read-only t))
+  (save-excursion
+    (set-buffer dtk-scratch-buffer )
+    (erase-buffer)
+    (goto-char (point-min))
+    (loop  for element in text
+           do
+           (insert
+            (format "%s\n" element)))
+    (setq contents (buffer-string)))
+  (dtk-speak contents)))
+
       
-  
-  
-(defun dtk-letter (letter)
+(defsubst dtk-letter (letter)
   "Speak a letter"
   (declare (special dtk-speaker-process
+                    dtk-speak-driver-initialized
                     dtk-quiet ))
   (unless dtk-quiet
-          (process-send-string dtk-speaker-process
-                       (format "dectalk_letter \%s\n" letter ))))
+          (when dtk-speak-driver-initialized
+               (process-send-string dtk-speaker-process
+                       (format "l {%s}\n" letter )))))
 
                                         ;Say these words, ie speak each of them as a word.
                                         ; Text is sent via the appropriate TCLSH. 
 (defun dtk-say (words)
   "Say these words"
   (declare (special dtk-speaker-process dtk-stop-immediately
-                    dtk-punctuation-mode  dtk-quiet ))
+                    dtk-speak-driver-initialized dtk-quiet))
   ;; I wont talk if you dont want me to
   (unless dtk-quiet 
-  (or (eq 'run (process-status dtk-speaker-process ))
-    (dtk-initialize) )
-  (and dtk-stop-immediately (dtk-stop ))
-  (let ((dtk-scratch-buffer (get-buffer-create "*dtk-scratch-buffer*"))
-        (mode  dtk-punctuation-mode ))
-    (save-excursion
-      (set-buffer dtk-scratch-buffer )
-      (let ((inhibit-read-only t))
-        (erase-buffer)
-      (insert words )
-      (delete-invisible-text)
-      (dtk-quote mode))
+    (or (eq 'run (process-status dtk-speaker-process ))
+        (dtk-initialize))
+    (when dtk-speak-driver-initialized
+      (and dtk-stop-immediately (dtk-stop ))
       (process-send-string dtk-speaker-process
-                           (format "dectalk_say { %s }  \n"
-                                   (buffer-string)))))))
-
-
-(defun dtk-tone (&optional  pitch duration wait)
-  "Produce a tone. Optional args: pitch 400 duration 50 wait nil  "
-  (declare (special dtk-quiet ))
-  (unless dtk-quiet 
-  (or pitch (setq pitch 440))
-  (or duration (setq duration 50 ))
-  (declare (special dtk-speaker-process   )) 
-  (process-send-string dtk-speaker-process
-                       (format "dectalk_tone   %d   %d\n"
-                               pitch duration ))
-  (dtk-force)
-  (when  wait (accept-process-output dtk-speaker-process 0 5)
-         (dtk-force)
-         (when wait
-           (accept-process-output dtk-speaker-process )))))
+                           (format "dectalk_say {%s}\n"
+                                    words)))))
 
 ;;}}}
 ;;{{{  sending commands 
@@ -571,18 +612,22 @@ only speak upto the first ctrl-m. "
 Interactive prefix arg means set   the global default value, and then set the
 current local  value to the result. "
   (interactive "nEnter new rate:\nP")
-  (declare (special dtk-speech-rate dtk-speaker-process))
-  (cond
-   (prefix
+  (declare (special dtk-speech-rate dtk-speaker-process
+                    dtk-speak-driver-initialized))
+  (when dtk-speak-driver-initialized
+    (cond
+     (prefix
       (setq-default dtk-speech-rate rate )
-  (setq dtk-speech-rate rate))
-   (t (make-local-variable 'dtk-speech-rate )
-      (setq dtk-speech-rate rate)))
-  (process-send-string dtk-speaker-process
-                       (format "dectalk_set_speech_rate %s; \
-dectalk_speak {set speaking rate to %s %s}\n"
-                               rate
-                               rate (if prefix "" " locally"))))
+      (setq dtk-speech-rate rate))
+     (t (make-local-variable 'dtk-speech-rate )
+        (setq dtk-speech-rate rate)))
+    (process-send-string dtk-speaker-process
+                         (format "dectalk_set_speech_rate %s\n"
+                                 rate))
+                         (message "Set speech rate to %s %s\n"
+                                  rate
+                                  (if prefix "" "locally"))))
+
 (defvar dtk-speech-rate-step 50 
   "* Determines step size used when setting speech rate via command
 dtk-set-predefined-speech-rate.  Formula used is
@@ -619,20 +664,21 @@ speaking characters.
 Interactive prefix arg means set   the global default value, and then set the
 current local  value to the result. "
   (interactive "nEnter new factor:\nP")
-  (declare (special dtk-character-scale dtk-speaker-process ))
-  (cond
-   (prefix
-    (setq-default dtk-character-scale factor)
-    (setq dtk-character-scale factor))
-   (t (make-local-variable 'dtk-character-scale)
-      (setq dtk-character-scale factor)))
-  (process-send-string dtk-speaker-process
-                       (format "dectalk_set_character_scale %s; \
-dectalk_speak {Set character scale factor to %s %s}\n"
-                               dtk-character-scale dtk-character-scale
-                               (if  prefix ""  "locally"))))
-
-
+  (declare (special dtk-character-scale dtk-speaker-process
+                    dtk-speak-driver-initialized))
+  (when dtk-speak-driver-initialized
+    (cond
+     (prefix
+      (setq-default dtk-character-scale factor)
+      (setq dtk-character-scale factor))
+     (t (make-local-variable 'dtk-character-scale)
+        (setq dtk-character-scale factor)))
+    (process-send-string dtk-speaker-process
+                         (format "dectalk_set_character_scale %s\n"
+                                 dtk-character-scale))
+    (message "Set character scale factor to %s %s"
+             dtk-character-scale
+             (if  prefix ""  "locally"))))
 
 (defun dtk-toggle-quiet (&optional prefix )
   "Toggle state of the speech device between being quiet and talkative.
@@ -669,7 +715,7 @@ type."
    (t (make-local-variable 'dtk-stop-immediately-while-typing)
     (setq dtk-stop-immediately-while-typing 
             (not dtk-stop-immediately-while-typing ))))
-  (message "%s turned %s immediate flushing of speech when typing. "
+  (message "%s turned %s immediate flushing of speech when typing "
            (if prefix "" " locally")
            (if dtk-stop-immediately-while-typing "on" "off" )))
 
@@ -679,10 +725,11 @@ type."
 Hungarian notation in program source code.  Interactive prefix arg
 means toggle the global default value, and then set the current local
 value to the result. "
-
   (interactive "P")
-  (declare (special dtk-speaker-process dtk-split-caps))
-  (cond
+  (declare (special dtk-speaker-process dtk-split-caps
+                    dtk-speak-driver-initialized))
+  (when dtk-speak-driver-initialized
+    (cond
    (prefix
     (setq-default  dtk-split-caps
                    (not  (default-value 'dtk-split-caps )))
@@ -693,30 +740,58 @@ value to the result. "
   (process-send-string dtk-speaker-process
                        (format "dectalk_split_caps %s\n"
                                (if dtk-split-caps 1 0 )))
-  (message "Turned %s split caps mode%s. "
+  (message "Turned %s split caps mode%s "
            (if dtk-split-caps "on" "off" )
-           (if prefix "" " locally")))
+           (if prefix "" " locally"))))
 
 (defun dtk-toggle-capitalization  (&optional prefix)
   "Toggle capitalization.  when set, capitalization is indicated by a
 short beep.  Interactive prefix arg means toggle the global default
 value, and then set the current local value to the result. "
   (interactive "P")
-  (declare (special dtk-speaker-process dtk-capitalize))
-  (cond
-   (prefix
-    (setq-default  dtk-capitalize
-                   (not  (default-value 'dtk-capitalize )))
-    (setq dtk-capitalize (default-value 'dtk-capitalize )))
-   (t (make-local-variable 'dtk-capitalize)
-      (setq dtk-capitalize 
-            (not dtk-capitalize ))))
-  (process-send-string dtk-speaker-process 
-                       (format "dectalk_capitalize  %s\n"
-                               (if dtk-capitalize  1 0 )))
-  (message "Turned %s capitalization  mode%s. "
-           (if dtk-capitalize  "on" "off" )
-           (if prefix "" " locally")))
+  (declare (special dtk-speaker-process dtk-capitalize
+                    dtk-speak-driver-initialized))
+  (when dtk-speak-driver-initialized
+    (cond
+     (prefix
+      (setq-default  dtk-capitalize
+                     (not  (default-value 'dtk-capitalize )))
+      (setq dtk-capitalize (default-value 'dtk-capitalize )))
+     (t (make-local-variable 'dtk-capitalize)
+        (setq dtk-capitalize 
+              (not dtk-capitalize ))))
+    (process-send-string dtk-speaker-process 
+                         (format "dectalk_capitalize  %s\n"
+                                 (if dtk-capitalize  1 0 )))
+    (message "Turned %s capitalization  mode%s "
+             (if dtk-capitalize  "on" "off" )
+             (if prefix "" " locally"))))
+
+
+(defun dtk-toggle-allcaps-beep  (&optional prefix)
+  "Toggle allcaps-beep.  when set, allcaps words  are  indicated by a
+short beep.  Interactive prefix arg means toggle the global default
+value, and then set the current local value to the result.
+Note that allcaps-beep is a very useful thing when programming.
+However it is irritating to have it on when reading documents."
+  (interactive "P")
+  (declare (special dtk-speaker-process dtk-allcaps-beep
+                    dtk-speak-driver-initialized))
+  (when dtk-speak-driver-initialized
+    (cond
+     (prefix
+      (setq-default  dtk-allcaps-beep
+                     (not  (default-value 'dtk-allcaps-beep )))
+      (setq dtk-allcaps-beep (default-value 'dtk-allcaps-beep )))
+     (t (make-local-variable 'dtk-allcaps-beep)
+        (setq dtk-allcaps-beep 
+              (not dtk-allcaps-beep ))))
+    (process-send-string dtk-speaker-process 
+                         (format "dectalk_allcaps_beep  %s\n"
+                                 (if dtk-allcaps-beep  1 0 )))
+    (message "Turned %s allcaps-beep  mode%s "
+             (if dtk-allcaps-beep  "on" "off" )
+             (if prefix "" " locally"))))
 
 (defun dtk-set-punctuations  (mode &optional prefix ) 
   "Set punctuation state.
@@ -731,17 +806,21 @@ current local  value to the result. "
                       t)
     current-prefix-arg))
   (declare (special dtk-punctuation-mode dtk-speaker-process
+                    dtk-speak-driver-initialized
                     dtk-punctuation-mode-alist))
-  (cond
-   (prefix
-    (setq dtk-punctuation-mode mode )
+  (when dtk-speak-driver-initialized
+    (cond
+     (prefix
+      (setq dtk-punctuation-mode mode )
       (setq-default dtk-punctuation-mode mode))
-   (t (make-local-variable 'dtk-punctuation-mode)
-      (setq dtk-punctuation-mode mode )))
-  (process-send-string dtk-speaker-process  
-                       (format "dectalk_set_punctuations %s;\
- dectalk_speak {set punctuation mode to %s}\n"
-                               mode mode )))
+     (t (make-local-variable 'dtk-punctuation-mode)
+        (setq dtk-punctuation-mode mode )))
+    (process-send-string dtk-speaker-process  
+                         (format "dectalk_set_punctuations %s\n"
+                                 mode))
+    (message "set punctuation mode to %s %s"
+             mode
+             (if prefix "" "locally"))))
 
 (defun dtk-set-punctuations-to-all (&optional prefix )
   "Set punctuation  mode to all.
@@ -787,9 +866,57 @@ europe, spell', all of which can be turned on or off. "
   "Restore sanity to the Dectalk.
  Typically used after the Dectalk has been power   cycled."
   (interactive)
-  (declare (special  dtk-speaker-process))
-  (process-send-string dtk-speaker-process
-                       (format "dectalk_reset \n")))
+  (declare (special  dtk-speaker-process
+                     dtk-speak-driver-initialized))
+  (when dtk-speak-driver-initialized
+    (process-send-string dtk-speaker-process
+                         (format "dectalk_reset \n"))))
+
+;;}}}
+;;{{{  pause and resume 
+(defvar dtk-paused nil
+  "Records if speech has been paused.")
+
+(defun dtk-pause (&optional prefix)
+  "Pause ongoing speech.  The speech can be resumed with command dtk-resume
+normally bound to \\[dtk-resume].  Pausing speech is useful when one needs to
+perform a few actions before continuing to read a large document.  Emacspeak
+gives you speech feedback as usual once speech has been paused.  dtk-resume
+continues the interrupted speech irrespective of the buffer in which it is
+executed. Optional first  arg pauses speech even if there
+is no ongoing speech. This lets you flush paused speech without having to resume it."
+  (interactive "P")
+  (declare (special dtk-speaker-process
+                    dtk-paused))
+  (when (or  prefix
+             (interactive-p)
+             (not dtk-paused))
+    (process-send-string dtk-speaker-process "dectalk_pause\n")
+    (setq dtk-paused t)
+    (emacspeak-auditory-icon 'button)))
+
+(defvar dtk-resume-should-toggle nil
+  "*T means dtk-resume acts as a toggle.")
+
+
+(defun dtk-resume ()
+  "Resumes  speech that has been suspended by executing
+command dtk-pause bound to \\[dtk-pause].
+If speech has not been paused,
+and variable dtk-resume-should-toggle is t
+ then this command will pause ongoing speech."
+  (interactive)
+  (declare (special dtk-speaker-process
+                    dtk-resume-should-toggle
+                    dtk-paused))
+  (cond
+   ((and dtk-resume-should-toggle
+         (not dtk-paused) )
+    (dtk-pause))
+   (t (process-send-string dtk-speaker-process "dectalk_resume\n")
+      (emacspeak-auditory-icon 'button)
+      (setq dtk-paused nil))))
+
 
 ;;}}}
 (provide 'dtk-tcl)
