@@ -1,19 +1,20 @@
-;;; $Id: emacspeak-eterm.el,v 5.0 1996/11/22 18:04:26 raman Exp $
+;;; $Id: emacspeak-eterm.el,v 7.0 1997/11/13 15:32:30 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak extension to speech enable eterm. 
 ;;; Keywords: Emacspeak, Eterm, Terminal emulation, Spoken Output
 ;;{{{  LCD Archive entry: 
+
 ;;; LCD Archive Entry:
-;;; emacspeak| T. V. Raman |raman@crl.dec.com 
+;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $date: $ |
-;;;  $Revision: 5.0 $ | 
+;;; $Date: 1997/11/13 15:32:30 $ |
+;;;  $Revision: 7.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-
+;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -49,8 +50,10 @@
 (eval-when (compile)
   (require 'emacspeak-fix-interactive)
   (condition-case nil
-      (require 'term)
-    ((error nil ))))
+      (progn
+        (require 'term)
+        (require 'tshell))
+    (error nil )))
 
 ;;}}}
 ;;{{{  Introduction:
@@ -69,7 +72,7 @@
 (defvar emacspeak-eterm-prefix "\C-t"
   "Prefix char used by emacspeak for navigating an eterm")
 
- (defun emacspeak-eterm-setup-keys()
+(defun emacspeak-eterm-setup-keys()
    "Make eterm usable with emacspeak"
    (declare (special emacspeak-prefix emacspeak-eterm-prefix
                      emacspeak-eterm-keymap  term-mode-map ))
@@ -112,8 +115,10 @@
    (define-key emacspeak-eterm-keymap ">" 'emacspeak-eterm-pointer-to-bottom)
    (define-key emacspeak-eterm-keymap "g" 'emacspeak-eterm-goto-line)
    (define-key emacspeak-eterm-keymap "s" 'emacspeak-eterm-search-forward )
-   (define-key emacspeak-eterm-keymap "r" 'emacspeak-eterm-search-backward)
-   (define-key emacspeak-eterm-keymap "y" 'emacspeak-eterm-kill-ring-save-region)
+   ;(define-key emacspeak-eterm-keymap "r" 'emacspeak-eterm-search-backward)
+   (define-key emacspeak-eterm-keymap "y"
+     'emacspeak-eterm-kill-ring-save-region)
+   (define-key emacspeak-eterm-keymap "x" 'emacspeak-eterm-copy-region-to-register)
    (define-key emacspeak-eterm-keymap "m" 'emacspeak-eterm-set-marker )
    (define-key emacspeak-eterm-keymap "\C-p"
      'emacspeak-eterm-toggle-pointer-mode)
@@ -123,11 +128,19 @@
    (define-key emacspeak-eterm-keymap "\C-d" 'emacspeak-eterm-describe-window)
    (define-key emacspeak-eterm-keymap "\C-m" 'emacspeak-eterm-speak-window)
    (define-key emacspeak-eterm-keymap "r" 'emacspeak-eterm-toggle-review)
-   (define-key emacspeak-eterm-keymap "q" 'emacspeak-eterm-toggle-review))
+   (define-key emacspeak-eterm-keymap "q" 'emacspeak-eterm-toggle-review)
+   (and term-raw-escape-map
+        (mapcar
+    (function 
+     (lambda (key)
+       (define-key term-raw-escape-map key 
+         (lookup-key (current-global-map) key))))
+    '("\M-x" "\C-h")))
+   t)
 
 (defvar emacspeak-eterm-raw-prefix
-  "\C-r"
-"Prefix key to use  to send out raw term input. 
+"\C-r"
+  "Prefix key to use  to send out raw term input. 
 Useful when eterm is in review mode. ")
 
 (defun emacspeak-eterm-setup-raw-keys ()
@@ -187,7 +200,7 @@ Execute end of line if
   "Speak cursor position"
   (interactive)
   (message
-   "Cursor at Row %s, Column %s"
+   "Cursor at Row %s Column %s"
    (term-current-row)
    (term-current-column)))
 
@@ -236,13 +249,14 @@ the Emacspeak pointer to be spoken. "
     (goto-char emacspeak-eterm-pointer)
     (emacspeak-speak-word nil )))
 
-(defun emacspeak-eterm-speak-pointer-char ()
-  "Speak the character  the pointer is on"
-  (interactive)
+(defun emacspeak-eterm-speak-pointer-char (&optional prefix)
+  "Speak char under eterm pointer.
+Pronounces character phonetically unless  called with a prefix arg."
+  (interactive "P")
   (declare (special emacspeak-eterm-pointer ))
   (save-excursion
     (goto-char emacspeak-eterm-pointer)
-    (emacspeak-speak-char t)))
+    (emacspeak-speak-char prefix)))
 
 ;;}}}
 ;;{{{  moving the screen pointer: 
@@ -270,7 +284,7 @@ the Emacspeak pointer to be spoken. "
 (defun emacspeak-eterm-pointer-to-bottom  () 
   "Move the pointer to the bottom  of the screen. "
   (interactive)
-  (declare (special term-home-marker emacspeak-eterm-pointer ))
+  (declare (special  emacspeak-eterm-pointer ))
   (save-excursion
     (goto-char (point-max))
     (set-marker emacspeak-eterm-pointer (point ))
@@ -447,7 +461,7 @@ If found, the Emacspeak pointer is left at the hit. "
                 (emacspeak-auditory-icon 'search-hit)
                 (emacspeak-eterm-speak-pointer-line))
          (t(emacspeak-auditory-icon 'search-miss)
-          (message "%s not found. " string )))))))
+          (message "%s not found " string )))))))
 
 ;;}}}
 ;;{{{  Highlight tracking:
@@ -470,7 +484,7 @@ This allows you to move between highlighted regions of the screen. "
        (found (set-marker emacspeak-eterm-pointer found )
               (emacspeak-auditory-icon 'large-movement)
               (emacspeak-eterm-speak-pointer-line))
-       (t (message "No color change found on the screen. "))))))
+       (t (message "No color change found on the screen "))))))
 
 (defun emacspeak-eterm-pointer-to-previous-color-change  (&optional count)
   "Move the eterm pointer to the next color change.
@@ -488,7 +502,7 @@ This allows you to move between highlighted regions of the screen. "
        (found (set-marker emacspeak-eterm-pointer found )
               (emacspeak-auditory-icon 'large-movement)
               (emacspeak-eterm-speak-pointer-line))
-       (t (message "No color change found on the screen. "))))))
+       (t (message "No color change found on the screen "))))))
 
 ;;}}}
 ;;{{{  reviewing the terminal:
@@ -515,15 +529,15 @@ without sending input to the terminal itself. "
   (if eterm-char-mode 
       (cond 
        (emacspeak-eterm-review-p        ;turn it off 
-        (message "Returning to terminal character mode. ")
+        (message "Returning to terminal character mode ")
         (setq emacspeak-eterm-review-p nil)
         (use-local-map term-raw-map))
        (t                               ; turn it on
-        (message "Entering terminal review mode, press  q  to return to normal")
+        (message "Entering terminal review mode press  q  to return to normal")
         (setq emacspeak-eterm-review-p t)
         (use-local-map emacspeak-eterm-keymap)))
     (message "Terminal review should be used when eterm is in
-character mode. "))
+character mode "))
   (emacspeak-auditory-icon
    (if emacspeak-eterm-review-p
        'on
@@ -564,10 +578,27 @@ emacspeak eterm pointer. "
   (kill-ring-save (marker-position emacspeak-eterm-marker )
                   (marker-position emacspeak-eterm-pointer))
   (emacspeak-auditory-icon 'mark-object)
-  (message "Snarfed %s characters. "
+  (message "Snarfed %s characters "
            (abs (- (marker-position emacspeak-eterm-marker )
                    (marker-position emacspeak-eterm-pointer)))))
 
+(defun emacspeak-eterm-copy-region-to-register  (register)
+  "Copy region delimited by the emacspeak eterm marker 
+set by command \\[emacspeak-eterm-set-marker] and the 
+emacspeak eterm pointer to a register."
+  (interactive "cCopy to register:")
+  (declare (special emacspeak-eterm-marker 
+                    emacspeak-eterm-pointer ))
+  (copy-to-register register 
+   (marker-position emacspeak-eterm-marker)
+                  (marker-position emacspeak-eterm-pointer)
+                  nil)
+  (emacspeak-auditory-icon 'mark-object)
+  (message "Snarfed %s characters to register %c "
+           (abs (- (marker-position emacspeak-eterm-marker )
+                   (marker-position emacspeak-eterm-pointer)))
+           register))
+(emacspeak-fix-interactive-command-if-necessary 'emacspeak-eterm-copy-region-to-register)
 ;;}}}
 ;;{{{  Defining and speaking terminal windows:
 
@@ -614,16 +645,13 @@ emacspeak eterm pointer. "
 (defun emacspeak-eterm-coordinates-to-position (coordinates)
   "Translate  screen coordinates specified
 as a cons cell (column . row ) to a buffer position in the eterm buffer"
-  (declare (special term-home-marker
-                    term-vertical-motion))
+  (declare (special term-home-marker))
   (let ((column (car coordinates ))
         (row (cdr coordinates )))
     (save-excursion
       (save-restriction
         (emacspeak-eterm-nuke-cached-info)
         (narrow-to-region term-home-marker (point-max ))
-        ;(term-vertical-motion (- row (term-current-row )))
-        ;(term-move-columns (-  column (current-column )))
         (term-goto row column )
         (emacspeak-eterm-nuke-cached-info)
         (point )))))
@@ -632,18 +660,18 @@ as a cons cell (column . row ) to a buffer position in the eterm buffer"
 ;;; returns a cons cell (column . row )
 (defun emacspeak-eterm-position-to-coordinates (position)
   "Translate a buffer position in the eterm buffer to screen coordinates"
-  (declare (special term-home-marker
-                    term-vertical-motion))
+  (declare (special term-home-marker))
   (save-excursion
     (save-restriction
       (narrow-to-region term-home-marker (point-max ))
-    (goto-char position)
-    (emacspeak-eterm-nuke-cached-info)
-    (let ((coordinates (cons
-     (term-current-column)
-     (term-current-row ))))
+      (goto-char position)
       (emacspeak-eterm-nuke-cached-info)
-      coordinates ))))
+      (let ((coordinates (cons
+                          (term-current-column)
+                          (term-current-row ))))
+        (emacspeak-eterm-nuke-cached-info)
+        coordinates ))))
+
 ;;; return contents of a term window 
 (defun emacspeak-eterm-return-window-contents (eterm-window)
   "Return  the contents of a window as a string"
@@ -865,7 +893,7 @@ activity in that area of the screen."
          (assert (numberp window-id) t
                  "Please specify a valid window id, a non-negative integer ")
     (setq emacspeak-eterm-focus-window window-id )
-    (message "Set emacspeak eterm's focus window  to %d " window-id )))))
+    (message "Set emacspeak eterm focus window  to %d " window-id )))))
 
 (defun emacspeak-eterm-speak-predefined-window ()
   "Speak a predefined eterm window between 1 and 10."
@@ -896,6 +924,7 @@ available with the prefix emacspeak-eterm-prefix and are listed below:
   (make-local-variable 'eterm-current-personality)
   (setq eterm-current-personality 'eterm-default-personality)
   (voice-lock-mode 1)
+  (modify-syntax-entry 10 ">")
   (make-local-variable 'emacspeak-eterm-pointer)
   (setq emacspeak-eterm-pointer  (copy-marker (point)))
   (make-local-variable 'emacspeak-eterm-marker)
@@ -1105,8 +1134,7 @@ there is terminal activity."
   (setq eterm-char-mode nil 
         eterm-line-mode t )
   (when (interactive-p)
-    (dtk-speak "Terminal line mode. "))
-  )
+    (dtk-speak "Terminal line mode ")))
 
 (defadvice term-char-mode (after emacspeak pre act)
   "Announce you entered character mode. "
@@ -1120,13 +1148,10 @@ there is terminal activity."
         eterm-line-mode nil )
   (emacspeak-eterm-setup-raw-keys)
   (when (interactive-p)
-    (dtk-speak "Terminal character mode. "))
-  )
+    (dtk-speak "Terminal character mode ")))
 
 ;;}}}
-;;{{{  Advice term functions that duplicate functionality of their comint
-
-counterparts
+;;{{{  Advice term functions that duplicate functionality of their comint counterparts
 
 (defadvice term-next-input (after emacspeak pre act)
   "Speak the line. "
@@ -1171,7 +1196,7 @@ counterparts
     (emacspeak-speak-line 1))))
 (defadvice term-dynamic-list-input-ring (after emacspeak pre act)
   "Provide auditory feedback"
-  (message  "Switch to the other window to browse the input history. "))
+  (message  "Switch to the other window to browse the input history "))
 
 (defadvice term-kill-output (after emacspeak pre act )
   "Provide auditory feedback"
@@ -1208,6 +1233,70 @@ counterparts
     (message "Switch to the completions window to browse the possible
 completions for filename at point")))
 
+;;}}}
+;;{{{  Launch remote terminals
+(defvar emacspeak-eterm-remote-hosts-table
+  (make-vector 127 0)
+  "obarray used for completing hostnames when prompting for a remote
+host. Hosts are added whenever a new hostname is encountered, and the
+list of known hostnames is persisted in file named by
+emacspeak-eterm-remote-hostnames")
+
+(defvar emacspeak-eterm-remote-hosts-cache
+  (expand-file-name "~/.emacspeak-hosts")
+  "File where list of known remote hosts is cached")
+
+(defun emacspeak-eterm-load-remote-hosts-cache ()
+  "Load cached remote hostnames"
+  (declare (special emacspeak-eterm-remote-hosts-table
+                    emacspeak-eterm-remote-hosts-cache))
+  (when (file-exists-p emacspeak-eterm-remote-hosts-cache )
+    (let ((host nil)
+          (hosts (find-file-noselect emacspeak-eterm-remote-hosts-cache)))
+      (save-excursion
+        (set-buffer hosts)
+        (goto-char (point-min))
+        (while (not (eobp))
+          (setq host
+                (buffer-substring
+                 (point)
+                 (progn (end-of-line) (point))))
+            (intern host emacspeak-eterm-remote-hosts-table)
+            (forward-line 1))))))
+
+(eval-when (load)
+  (emacspeak-eterm-load-remote-hosts-cache))
+
+(defun emacspeak-eterm-cache-remote-host (host)
+  "Add this hostname to cache of remote hostnames"
+(declare (special emacspeak-eterm-remote-hosts-table
+                    emacspeak-eterm-remote-hosts-cache))
+(let ((buffer (find-file-noselect
+               emacspeak-eterm-remote-hosts-cache)))
+  (save-excursion
+    (set-buffer buffer)
+    (goto-char (point-max))
+    (insert (format "%s\n" host))
+            (save-buffer))
+(intern host emacspeak-eterm-remote-hosts-table)))
+
+(defun emacspeak-eterm-remote-term (host )
+  "Start a terminal-emulator in a new buffer."
+  (interactive
+   (list
+    (completing-read "Remote host: "
+                     emacspeak-eterm-remote-hosts-table)))
+  (declare (special emacspeak-eterm-remote-hosts-table))
+  (require 'term)
+  (require 'tshell)
+  (set-buffer (make-term (format "%s-terminal" host)
+                         "rlogin"nil  host))
+  (tshell-mode)
+  (term-char-mode)
+  (unless (intern-soft host emacspeak-eterm-remote-hosts-table)
+    (emacspeak-eterm-cache-remote-host host))
+  (switch-to-buffer (format "*%s-terminal*" host) ))
+     
 ;;}}}
  (provide 'emacspeak-eterm)
 ;;{{{  emacs local variables
