@@ -1,19 +1,20 @@
-;;; $Id: emacspeak-gnus.el,v 5.0 1996/11/22 18:04:26 raman Exp $
+;;; $Id: emacspeak-gnus.el,v 7.0 1997/11/13 15:32:30 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak extension to speech enable Gnus
 ;;; Keywords: Emacspeak, Gnus, Advice, Spoken Output, News
 ;;{{{  LCD Archive entry: 
+
 ;;; LCD Archive Entry:
-;;; emacspeak| T. V. Raman |raman@crl.dec.com 
+;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $date: $ |
-;;;  $Revision: 5.0 $ | 
+;;; $Date: 1997/11/13 15:32:30 $ |
+;;;  $Revision: 7.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-
+;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -37,8 +38,8 @@
 (require 'cl)
 (declaim  (optimize  (safety 0) (speed 3)))
 (eval-when (compile) (require 'gnus)
-           (require 'voice-lock))
-(require 'advice)
+           (require 'voice-lock)
+           (require 'emacspeak-fix-interactive))
 (require 'emacspeak-speak)
 
 (require 'emacspeak-sounds)
@@ -64,7 +65,8 @@
                       (define-key gnus-summary-mode-map "\C-t"
                         'gnus-summary-toggle-header)
                       (define-key gnus-summary-mode-map "T" 'gnus-summary-hide-all-headers )
-                      (define-key gnus-summary-mode-map "t" 'gnus-summary-show-some-headers)
+                      (define-key gnus-summary-mode-map "t"
+                        'gnus-summary-show-some-headers)
                       (define-key gnus-summary-mode-map '[left] 'gnus-summary-catchup-quietly-and-exit)
                       (define-key gnus-summary-mode-map '[right] 'gnus-summary-show-article))))
 
@@ -91,7 +93,7 @@
   "Article headers to ignore when only important article headers are to be
 spoken.
 See command \\[gnus-summary-show-some-headers].")
-
+(declaim (special gnus-ignored-headers))
 (setq gnus-ignored-headers "^.*:")
 (declaim (special gnus-visible-headers))
 (setq gnus-visible-headers "^Subject:")
@@ -123,15 +125,21 @@ reading news."
   (dtk-speak (gnus-summary-subject-string )))
 
 ;;}}}
-
 ;;{{{  starting up:
 
+(defadvice gnus-group-post-news (after emacspeak pre act comp)
+  "Provide auditory feedback"
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'open-object)
+    (emacspeak-speak-line)))
+
+
 (defadvice gnus-group-get-new-news (around emacspeak pre act )
   "Temporarily deactivate advice on message"
   (dtk-speak  "Getting new  gnus")
     (let ((emacspeak-speak-messages nil ))
     ad-do-it)
-    (message "Gnus is ready. ")
+    (message "Gnus is ready ")
     (emacspeak-auditory-icon 'news))
 
 ;;}}}
@@ -208,7 +216,7 @@ this group is being deselected."
       (let ((saved-point (point )))
         ad-do-it
         (if (= saved-point (point))
-            (dtk-speak "No more articles. ")
+            (dtk-speak "No more articles ")
           (progn 
             (emacspeak-auditory-icon 'select-object ))
           (dtk-speak (gnus-summary-subject-string )))
@@ -222,11 +230,10 @@ Produce an auditory icon if possible."
       (let ((saved-point (point )))
         ad-do-it
         (if (= saved-point (point))
-            (dtk-speak "No more articles. ")
+            (dtk-speak "No more articles ")
           (progn 
             (emacspeak-auditory-icon 'select-object )
-            (dtk-speak (gnus-summary-subject-string ))))
-        ad-return-value )
+            (dtk-speak (gnus-summary-subject-string )))))
     ad-return-value ))
 
 (defadvice gnus-summary-prev-unread-subject  (around  emacspeak pre act)
@@ -236,7 +243,7 @@ Produce an auditory icon if possible."
       (let ((saved-point (point )))
         ad-do-it
         (if (= saved-point (point))
-            (dtk-speak "No more unread articles. ")
+            (dtk-speak "No more unread articles ")
           (progn 
             (emacspeak-auditory-icon 'select-object )
             (dtk-speak (gnus-summary-subject-string ))))
@@ -250,7 +257,7 @@ Produce an auditory icon if possible."
       (let ((saved-point (point )))
         ad-do-it
         (if (= saved-point (point))
-            (dtk-speak "No more articles. ")
+            (dtk-speak "No more articles ")
           (progn 
             (emacspeak-auditory-icon 'select-object )
             (dtk-speak (gnus-summary-subject-string )))))
@@ -264,7 +271,7 @@ Produce an auditory icon if possible."
         (let ((saved-point (point )))
           ad-do-it
           (if (= saved-point (point))
-              (dtk-speak "No more articles. ")
+              (dtk-speak "No more articles ")
             (progn 
               (emacspeak-auditory-icon 'select-object )
               (dtk-speak (gnus-summary-subject-string ))))
@@ -388,11 +395,9 @@ instead you hear only the first screenful.")
 (defsubst emacspeak-gnus-speak-article-body ()
   (declare (special emacspeak-gnus-large-article
                     voice-lock-mode dtk-punctuation-mode))
-  (let ((window nil ))
     (save-excursion
       (set-buffer  "*Article*")
       (goto-char (point-min))
-      (setq window (get-buffer-window "*Article*"))
       (setq dtk-punctuation-mode "some")
       (voice-lock-mode 1)
       (emacspeak-dtk-sync)
@@ -404,7 +409,7 @@ instead you hear only the first screenful.")
           (let ((start (point)))
             (move-to-window-line -1)
             (end-of-line)
-          (emacspeak-speak-region start (point))))))))
+          (emacspeak-speak-region start (point)))))))
            
            
 
@@ -507,8 +512,47 @@ instead you hear only the first screenful.")
   (when (interactive-p)
     (emacspeak-auditory-icon 'button)))
 
-;;}}}
 
+(defadvice gnus-article-goto-prev-page (after emacspeak pre act comp)
+  "Provide auditory feedback."
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'scroll)
+    (sit-for 1)
+    (emacspeak-speak-current-window)))
+
+(defadvice gnus-article-goto-next-page (after emacspeak pre act comp)
+  "Provide auditory feedback."
+  (when (interactive-p)
+    (emacspeak-auditory-icon 'scroll)
+    (sit-for 1)
+    (emacspeak-speak-current-window)))
+
+;;}}}
+;;{{{  fix interactive commands
+
+(loop for f in
+      '(gnus-fetch-group
+        gnus-group-apropos
+        gnus-group-description-apropos
+        gnus-group-enter-directory
+        gnus-group-kill-level
+        gnus-group-list-all-matching
+        gnus-group-list-level
+        gnus-group-list-matching
+        gnus-group-make-empty-virtual
+        gnus-group-mark-regexp
+        gnus-summary-import-article
+        gnus-summary-limit-to-author
+        gnus-summary-limit-to-marks
+        gnus-summary-limit-to-subject
+        gnus-summary-mark-above
+        gnus-summary-mark-below
+        gnus-summary-refer-article
+        gnus-summary-resend-message)
+      do
+      (emacspeak-fix-interactive-command-if-necessary f))
+
+;;}}}
 (provide 'emacspeak-gnus)
 ;;{{{  end of file 
 
