@@ -1,19 +1,20 @@
-;;; $Id: emacspeak-kotl.el,v 5.0 1996/11/22 18:04:26 raman Exp $
+;;; $Id: emacspeak-kotl.el,v 7.0 1997/11/13 15:32:30 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak extensions for Bob Weiner's excellent Outliner
 ;;; Keywords: Emacspeak, Speech Access, Hyperbole, Outliner
 ;;{{{  LCD Archive entry: 
+
 ;;; LCD Archive Entry:
-;;; emacspeak| T. V. Raman |raman@crl.dec.com 
+;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $date: $ |
-;;;  $Revision: 5.0 $ | 
+;;; $Date: 1997/11/13 15:32:30 $ |
+;;;  $Revision: 7.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-
+;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -158,8 +159,7 @@
   (when (interactive-p)
     (emacspeak-auditory-icon 'delete-object)
     (emacspeak-speak-line 1)
-    (dtk-tone 500 30)
-    (dtk-force)))
+    (dtk-tone 500 30)))
 (ad-set-advice-info 'kotl-mode:kill-word nil)
 (defadvice kotl-mode:kill-word (before emacspeak pre act )
   "Speak word before killing it. "
@@ -167,8 +167,7 @@
     (save-excursion
       (skip-syntax-forward " ")
     (emacspeak-speak-word 1 ))
-    (dtk-tone 500 30)
-    (dtk-force)))
+    (dtk-tone 500 30)))
 (ad-set-advice-info 'kotl-mode:backward-kill-word nil)
 (defadvice kotl-mode:backward-kill-word (before emacspeak pre act)
   "Speak word before killing it. "
@@ -178,8 +177,7 @@
         (forward-word -1)
         (emacspeak-speak-region (point) start )
         )
-      (dtk-tone 500 30)
-      (dtk-force))))
+      (dtk-tone 500 30))))
 
 (ad-set-advice-info 'kotl-mode:kill-sentence nil)
 (defadvice kotl-mode:kill-sentence (before emacspeak pre act )
@@ -187,8 +185,7 @@
   (when (interactive-p)
     (emacspeak-auditory-icon 'delete-object)
     (emacspeak-speak-line 1 )
-    (dtk-tone 500 30)
-    (dtk-force)))
+    (dtk-tone 500 30)))
 (ad-set-advice-info 'kotl-mode:kill-ring-save nil)
 (defadvice kotl-mode:kill-ring-save (after emacspeak pre act)
   "Indicate that region has been copied to the kill ring.
@@ -230,15 +227,13 @@
   "Speak character you're deleting. "
   (when (interactive-p )
     (emacspeak-speak-char t)
-    (dtk-tone 500 30)
-    (dtk-force)))
+    (dtk-tone 500 30)))
 (ad-set-advice-info 'kotl-mode:delete-backward-char nil)
 (defadvice kotl-mode:delete-backward-char (before emacspeak pre act)
   "Speak character you're deleting. " 
   (when (interactive-p )
     (emacspeak-speak-this-char (preceding-char ))
-    (dtk-tone 500 30)
-    (dtk-force)))
+    (dtk-tone 500 30)))
 (ad-set-advice-info 'kotl-mode:transpose-chars nil)
 (defadvice kotl-mode:transpose-chars (after emacspeak pre act )
   "Provide auditory feedback"
