@@ -1,19 +1,20 @@
-;;; $Id: emacspeak-speak.el,v 5.0 1996/11/22 18:04:26 raman Exp $
+;;; $Id: emacspeak-speak.el,v 7.0 1997/11/13 15:32:30 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Contains the functions for speaking various chunks of text
 ;;; Keywords: Emacspeak, Speak, Spoken Output
 ;;{{{  LCD Archive entry: 
+
 ;;; LCD Archive Entry:
-;;; emacspeak| T. V. Raman |raman@crl.dec.com 
+;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $date: $ |
-;;;  $Revision: 5.0 $ | 
+;;; $Date: 1997/11/13 15:32:30 $ |
+;;;  $Revision: 7.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-
+;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -34,7 +35,6 @@
 ;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 ;;}}}
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;{{{  Required modules
@@ -109,6 +109,78 @@ At present, we assume that region from start to end has the same personality. "
              (set-buffer-modified-p modification-flag ))))
      (setq inhibit-read-only nil))))
 
+;;}}}
+;;{{{  Apply audio annotations
+
+;;; prompt for auditory icon with completion
+
+(defsubst ems-ask-for-auditory-icon ()
+  (declare (special emacspeak-sounds-table))
+  (intern
+   (completing-read  "Select sound:"
+                  (mapcar
+                   (lambda (xx)
+                     (let ((x (car xx)))
+                     (list
+                      (format "%s" x)
+                      (format "%s" x ))))
+                   emacspeak-sounds-table))))
+
+(defun emacspeak-audio-annotate-paragraphs (&optional prefix)
+  "Set property auditory-icon at front of all paragraphs.
+Interactive prefix arg prompts for sound cue to use"
+  (interactive "P")
+  (save-excursion
+    (goto-char (point-max))
+    (ems-modify-buffer-safely
+     (let ((sound-cue  (if prefix
+                           (ems-ask-for-auditory-icon)
+                         'paragraph)))
+       (while (not (bobp))
+         (backward-paragraph)
+         (put-text-property  (1+ (point))
+                             (+ 2    (point ))
+                             'auditory-icon sound-cue ))))))
+
+(defvar  emacspeak-speak-paragraph-personality 'paul-animated
+  "*Personality for using to mark start of paragraph. ")
+
+(defvar emacspeak-speak-voice-annotated-paragraphs nil
+  "Records if paragraphs in this buffer have been voice annotated.")
+(make-variable-buffer-local
+ 'emacspeak-speak-voice-annotated-paragraphs)
+
+(defsubst emacspeak-speak-voice-annotate-paragraphs ()
+  "Locate paragraphs and voice annotate the first word.
+Here, paragraph is taken to mean a chunk of text preceeded by a blank line.
+Useful to do this before you listen to an entire buffer."
+  (interactive)
+  (declare (special emacspeak-speak-paragraph-personality
+                    emacspeak-speak-voice-annotated-paragraphs))
+  (when emacspeak-speak-paragraph-personality
+      (save-excursion
+    (goto-char (point-min))
+    (let ((start nil)
+          (blank-line "\n[ \t\n]*\n"))
+      (ems-modify-buffer-safely
+       (while (re-search-forward blank-line nil t)
+         (skip-syntax-forward " ")
+         (setq start (point))
+         (unless (get-text-property start 'personality)
+           (skip-syntax-forward "^ ")
+           (put-text-property start (point)
+                              'personality
+                              emacspeak-speak-paragraph-personality)))))
+    (setq emacspeak-speak-voice-annotated-paragraphs t))))
+
+;;}}}
+;;{{{ helper function --prepare completions buffer
+(defsubst emacspeak-prepare-completions-buffer()
+  (goto-char (point-min))
+        (forward-line 3)
+        (delete-region (point-min) (point))
+        (emacspeak-auditory-icon 'help))
+
 ;;}}}
 ;;{{{  Actions
 
@@ -132,6 +204,7 @@ is executed when the text unit at that point is spoken.")
 ;;; Return the appropriate action hook variable that defines actions
 ;;; for this mode.
 
+
 (defsubst  emacspeak-action-get-action-hook (mode)
   (intern (format "emacspeak-%s-actions-hook" mode )))
   
@@ -171,8 +244,6 @@ current local  value to the result. "
         (funcall  action-spec )
       (error (message "Invalid actionat %s" (point )))))))
 
-
-
 ;;}}}
 ;;{{{  line, Word and Character echo 
 
@@ -197,7 +268,7 @@ current local  value to the result. "
 	    (not emacspeak-line-echo ))))
   (emacspeak-auditory-icon
    (if emacspeak-line-echo 'on 'off))
-  (message "Turned %s line echo%s. "
+  (message "Turned %s line echo%s "
            (if emacspeak-line-echo "on" "off" )
 	   (if prefix "" " locally")))
 
@@ -222,7 +293,7 @@ current local  value to the result. "
 	    (not emacspeak-word-echo ))))
   (emacspeak-auditory-icon
    (if emacspeak-word-echo 'on 'off ))
-  (message "Turned %s word echo%s. "
+  (message "Turned %s word echo%s "
            (if emacspeak-word-echo "on" "off" )
 	   (if prefix "" " locally")))
 (defvar emacspeak-character-echo t
@@ -246,7 +317,7 @@ current local  value to the result. "
 	    (not emacspeak-character-echo ))))
   (emacspeak-auditory-icon
    (if emacspeak-character-echo 'on 'off))
-  (message "Turned %s character echo%s. "
+  (message "Turned %s character echo%s "
            (if emacspeak-character-echo "on" "off" )
 	   (if prefix "" " locally")))
 
@@ -256,7 +327,7 @@ current local  value to the result. "
 (defvar emacspeak-show-point nil 
   "If t, then emacspeak-speak-line indicates position of point by an
 aural highlight.  Do not set this variable by hand; Use
-\\[emacspeak-toggle-show-point]"
+command emacspeak-toggle-show-point bound to \\[emacspeak-toggle-show-point]"
 )
 
 (defun emacspeak-toggle-show-point (&optional prefix)
@@ -275,10 +346,22 @@ current local  value to the result. "
 	    (not emacspeak-show-point ))))
   (emacspeak-auditory-icon
    (if emacspeak-show-point 'on 'off))
-  (message "Turned %s show point %s. "
+  (message "Turned %s show point %s "
            (if emacspeak-show-point "on" "off" )
 	   (if prefix "" " locally")))
 
+;;}}}
+;;{{{ compute percentage into the buffer:
+
+(defsubst emacspeak-get-current-percentage-into-buffer ()
+  (let* ((pos (point))
+	 (total (buffer-size))
+	 (percent (if (> total 50000)
+		      ;; Avoid overflow from multiplying by 100!
+		      (/ (+ (/ total 200) (1- pos)) (max (/ total 100) 1))
+		    (/ (+ (/ total 2) (* 100 (1- pos))) (max total 1)))))
+    percent))
+
 ;;}}}
 ;;{{{  indentation:
 
@@ -293,8 +376,7 @@ emacspeak-toggle-audio-indentation bound to
 (defsubst emacspeak-indent (indent)
   (when (> indent 1 )
     (let ((duration (+ 50 (* 20  indent ))))
-      (dtk-tone  250 duration)
-      (dtk-force))))
+      (dtk-tone  250 duration))))
     
 
 (defvar emacspeak-audio-indentation-methods
@@ -343,22 +425,21 @@ results in the number of initial spaces being spoken."
   "Bring emacspeak and dtk in sync. "
   (declare (special dtk-speaker-process
                     dtk-punctuation-mode dtk-speech-rate dtk-character-scale
-                    dtk-capitalize dtk-split-caps ))
+                    dtk-capitalize dtk-split-caps
+                    dtk-allcaps-beep))
   (let ((command (concat 
                   (format "dectalk_set_punctuations %s  \n "
                           dtk-punctuation-mode )
                   (format "dectalk_capitalize %s \n "
                           (if dtk-capitalize 1  0 ))
+                  (format "dectalk_allcaps_beep %s \n "
+                          (if dtk-allcaps-beep 1  0 ))
                   (format "dectalk_split_caps  %s \n "
                           (if dtk-split-caps 1 0 ))
                   (format "dectalk_set_speech_rate %s \n" dtk-speech-rate)
                   (format "dectalk_set_character_scale %s \n "
-                          dtk-character-scale)
-                  )))
-    (process-send-string dtk-speaker-process command )
-    command
-    )
-  )
+                          dtk-character-scale))))
+    (process-send-string dtk-speaker-process command )))
 
 ;;}}}
 ;;{{{ functions: 
@@ -368,10 +449,39 @@ results in the number of initial spaces being spoken."
 (defsubst emacspeak-speak-region (start end )
   "Speak region. "
   (interactive "r" )
-  (emacspeak-handle-auditory-icon-at-point)
+  (declare (special emacspeak-speak-voice-annotated-paragraphs))
+  (when (and voice-lock-mode
+           (not emacspeak-speak-voice-annotated-paragraphs))
+(save-restriction  
+(narrow-to-region start end )
+(emacspeak-speak-voice-annotate-paragraphs)))
   (emacspeak-handle-action-at-point)
   (dtk-speak (buffer-substring start end )))
 
+(defvar emacspeak-horizontal-rule "^\\([=_-]\\)\\1+$"
+  "*Regular expression to match horizontal rules in ascii text. ")
+
+(put 'emacspeak-horizontal-rule 'variable-interactive
+     "sEnterregular expression to match horizontal rule: ")
+
+
+(defvar emacspeak-decoration-rule
+  "^[ \t!@#$%^&*()<>|_=+/\\,.;:-]+$"
+  "*Regular expressions to match lines that are purely decorative ascii")
+
+(put 'emacspeak-decoration-rule 'variable-interactive
+     "sEnterregular expression to match lines that are decorative ASCII: ")
+
+(defvar emacspeak-unspeakable-rule
+  "^[^0-9a-zA-Z]+$"
+  "*Regular expression to match lines containing only non-alphanumeric
+characters.
+emacspeak will generate a tone instead of speaking such lines when
+punctuation mode is set to some.")
+
+(put 'emacspeak-unspeakable-rule 'variable-interactive
+     "sEnterregular expression to match unspeakable lines: ")
+
 (defun emacspeak-speak-line (&optional arg)
   "Speak current line.  With prefix arg, speaks the rest of the line
   from point.  Negative prefix optional arg speaks from start of line
@@ -379,22 +489,27 @@ results in the number of initial spaces being spoken."
   indentation with a tone if audio indentation is in use.  Indicates
   position of point with an aural highlight if option
   emacspeak-show-point is turned on --see command emacspeak-show-point
-  bound to \\[emacspeak-show-point]."
+  bound to \\[emacspeak-show-point].
+Lines that start hidden blocks of text,
+e.g. outline header lines,
+or header lines of blocks created by command
+emacspeak-hide-or-expose-block are indicated with auditory icon ellipses."
   (interactive "P")
   (declare (special voice-lock-mode
-                    emacspeak-show-point 
-                    emacspeak-audio-indentation))
+                    emacspeak-show-point
+                    emacspeak-decoration-rule
+                    emacspeak-horizontal-rule
+                    emacspeak-unspeakable-rule emacspeak-audio-indentation))
   (when (listp arg) (setq arg (car arg )))
   (save-excursion 
     (let ((start  nil)
           (end nil )
+          (dtk-stop-immediately dtk-stop-immediately)
           (inhibit-point-motion-hooks t)
           (line nil)
           (orig (point)) 
-          (indent nil)
-          (dtk-stop-immediately t))
+          (indent nil))
       (beginning-of-line)
-      (emacspeak-handle-auditory-icon-at-point)
       (emacspeak-handle-action-at-point)
       (setq start (point))
       (when (and emacspeak-audio-indentation
@@ -415,16 +530,39 @@ results in the number of initial spaces being spoken."
         (setq dtk-stop-immediately nil )
         (emacspeak-indent indent ))
       (if emacspeak-show-point
-                  (ems-set-personality-temporarily
-                   (point) (1+ (point))
-                   'paul-animated 
-                   (setq line
-                         (buffer-substring  start end )))
-                (setq line (buffer-substring start end )))
+          (ems-set-personality-temporarily
+           (point) (1+ (point))
+           'paul-animated 
+           (setq line
+                 (buffer-substring  start end )))
+        (setq line (buffer-substring start end )))
+      (when (get-text-property  start 'emacspeak-hidden-block)
+        (emacspeak-auditory-icon 'ellipses))
       (cond
-       ((string= ""  (buffer-substring start end))
+       ((string= ""  (buffer-substring start end)) ;blank line
         (if dtk-stop-immediately (dtk-stop))
-        (dtk-tone 200   250))
+        (dtk-tone 200   250 'force))
+       ((string-match  "^[ \t]+$" (buffer-substring start end )) ;only white space
+        (if dtk-stop-immediately (dtk-stop))
+        (dtk-tone 250   250 'force))
+       ((and
+         (not (string= "all" dtk-punctuation-mode))
+         (string-match  emacspeak-horizontal-rule
+                        (buffer-substring start end))) ;horizontal rule
+        (if dtk-stop-immediately (dtk-stop))
+        (dtk-tone 350   250 'force))
+       ((and
+         (not (string= "all" dtk-punctuation-mode))
+         (string-match  emacspeak-decoration-rule
+                        (buffer-substring start end)) ) ;decorative rule
+        (if dtk-stop-immediately (dtk-stop))
+        (dtk-tone 450   250 'force))
+       ((and
+         (not (string= "all" dtk-punctuation-mode))
+         (string-match  emacspeak-unspeakable-rule
+                        (buffer-substring start end)) ) ;unspeakable rule
+        (if dtk-stop-immediately (dtk-stop))
+        (dtk-tone 550   250 'force))
        (t
         (if (and (string= "speak" emacspeak-audio-indentation )
                  (null arg )
@@ -444,7 +582,6 @@ Negative prefix arg speaks from start of word to point."
   (interactive "P")
   (declare (special voice-lock-mode))
   (when (listp arg) (setq arg (car arg )))
-  (emacspeak-handle-auditory-icon-at-point)
   (emacspeak-handle-action-at-point)
   (cond
    ((looking-at  "[ \t]+" ) (dtk-say " space "))
@@ -553,7 +690,6 @@ Pronounces character phonetically unless  called with a prefix arg."
   (let ((dtk-stop-immediately t )
         (char  (following-char )))
     (when char
-      (emacspeak-handle-auditory-icon-at-point)
       (emacspeak-handle-action-at-point)
       (cond
        ((and (not prefix)
@@ -561,23 +697,66 @@ Pronounces character phonetically unless  called with a prefix arg."
         (dtk-speak (emacspeak-get-phonetic-string char )))
        ((emacspeak-is-alpha-p char) (dtk-letter (char-to-string char )))
        (t (dtk-dispatch
-           (dtk-char-to-speech char ))
-(dtk-force))))))
+           (dtk-char-to-speech char )))))))
 
 (defun emacspeak-speak-this-char (char)
   "Speak this char. "
   (let ((dtk-stop-immediately t ))
     (when char
-      (emacspeak-handle-auditory-icon-at-point)
       (emacspeak-handle-action-at-point)
       (cond
        ((emacspeak-is-alpha-p char) (dtk-letter (char-to-string char )))
        (t (dtk-dispatch
-           (dtk-char-to-speech char ))
-(dtk-force))))))
+           (dtk-char-to-speech char )))))))
 
 
+;;{{{ emacspeak-speak-display-char
 
+(defun emacspeak-speak-display-char  (&optional prefix)
+  "Display char under point using current speech display table.
+Behavior is the same as command emacspeak-speak-char
+bound to \\[emacspeak-speak-char]
+for characters in the range 0--127."
+  (interactive "P")
+  (declare (special dtk-display-table ))
+  (let ((char (following-char )))
+    (cond
+     ((and dtk-display-table
+           (> char 127))
+      (dtk-dispatch (aref dtk-display-table char)))
+     (t (emacspeak-speak-char prefix)))))
+
+;;}}}
+;;{{{ emacspeak-speak-set-display-table
+
+(defvar emacspeak-speak-display-table-list 
+'(("iso ascii" . "iso ascii")
+("default" . "default"))
+"Available speech display tables.")
+
+(defun emacspeak-speak-set-display-table(&optional prefix)
+  "Sets up buffer specific speech display table that controls how
+special characters are spoken. Interactive prefix argument causes
+setting to be global."
+  (interactive "P")
+  (declare (special dtk-display-table
+                    emacspeak-speak-display-table-list))
+  (let ((type (completing-read
+               "Select speech display table: "
+               emacspeak-speak-display-table-list
+               nil t ))
+        (table nil))
+    (cond
+     ((string= "iso ascii" type)
+      (setq table dtk-iso-ascii-character-to-speech-table))
+     (t (setq table nil)))
+    (cond
+     (prefix
+      (setq-default dtk-display-table table )
+      (setq dtk-display-table table))
+     (t (setq dtk-display-table table)))))
+
+;;}}}
 (defun emacspeak-speak-sentence (&optional arg)
   "Speak current sentence.
 With prefix arg, speaks the rest of the sentence  from point.
@@ -593,7 +772,6 @@ Negative prefix arg speaks from start of sentence to point."
       (setq end (point))
       (backward-sentence 1)
       (setq start (point))
-      (emacspeak-handle-auditory-icon-at-point)
       (emacspeak-handle-action-at-point)
       (cond
        ((null arg ))
@@ -622,7 +800,6 @@ If voice-lock-mode is on, then uses the personality. "
           (backward-sexp 1)
         (error nil ))
       (setq start (point))
-      (emacspeak-handle-auditory-icon-at-point)
       (emacspeak-handle-action-at-point)
       (cond
        ((null arg ))
@@ -644,7 +821,6 @@ If voice-lock-mode is on, then it will use any defined personality. "
           (end nil))
       (mark-page)
       (setq start  (point))
-      (emacspeak-handle-auditory-icon-at-point)
       (emacspeak-handle-action-at-point)
       (setq end  (mark))
       (cond
@@ -670,7 +846,6 @@ If voice-lock-mode is on, then it will use any defined personality. "
       (setq end (point))
       (backward-paragraph 1)
       (setq start (point))
-      (emacspeak-handle-auditory-icon-at-point)
       (emacspeak-handle-action-at-point)
       (cond
        ((null arg ))
@@ -684,12 +859,17 @@ If voice-lock-mode is on, then it will use any defined personality. "
 (defun emacspeak-speak-buffer (&optional arg) 
   "Speak current buffer  contents.
 With prefix arg, speaks the rest of the buffer from point.
-Negative prefix arg speaks from start of buffer to point. "
+Negative prefix arg speaks from start of buffer to point.
+ If voice lock mode is on, the paragraphs in the buffer are
+voice annotated first,  see command emacspeak-speak-voice-annotate-paragraphs."
   (interactive "P" )
+  (declare (special emacspeak-speak-voice-annotated-paragraphs))
+  (when (and voice-lock-mode
+             (not emacspeak-speak-voice-annotated-paragraphs))
+    (emacspeak-speak-voice-annotate-paragraphs))
   (when (listp arg) (setq arg (car arg )))
   (let ((start nil )
         (end nil))
-    (emacspeak-handle-auditory-icon-at-point)
     (cond
      ((null arg)
       (setq start (point-min)
@@ -700,16 +880,29 @@ Negative prefix arg speaks from start of buffer to point. "
      (t (setq start (point-min)
               end (point))))
     (dtk-speak (buffer-substring start end ))))
+(defun emacspeak-speak-front-of-buffer()
+  "Speak   the buffer from start to   point"
+  (interactive)
+  (emacspeak-speak-buffer -1))
+
+(defun emacspeak-speak-rest-of-buffer()
+  "Speak remainder of the buffer starting at point"
+  (interactive)
+  (emacspeak-auditory-icon 'select-object)
+  (emacspeak-speak-buffer 1))
+
 (defun emacspeak-speak-help(&optional arg)
   "Speak help buffer if one present.
 With prefix arg, speaks the rest of the buffer from point.
 Negative prefix arg speaks from start of buffer to point."
   (interactive "P")
+(declare (special voice-lock-mode))
   (let ((help-buffer (get-buffer "*Help*")))
     (cond
      (help-buffer
       (save-excursion
 	(set-buffer help-buffer)
+          (voice-lock-mode 1)
 	(emacspeak-speak-buffer arg )))
      (t (dtk-speak "First ask for help" )))))
 
@@ -797,7 +990,7 @@ WIth prefix argument N, move N items (negative N means move backward)."
 (defun emacspeak-get-current-completion-from-completions  ()
   "Return the completion string under point in the
 *Completions* buffer."
-  (let (beg end completion)
+  (let (beg end)
     (if (and (not (eobp)) (get-text-property (point) 'mouse-face))
 	(setq end (point) beg (1+ (point))))
     (if (and (not (bobp)) (get-text-property (1- (point)) 'mouse-face))
@@ -808,7 +1001,7 @@ WIth prefix argument N, move N items (negative N means move backward)."
                (previous-single-property-change beg 'mouse-face)
                (point-min)))
     (setq end (or (next-single-property-change end 'mouse-face) (point-max)))
-    (setq completion (buffer-substring beg end))))
+     (buffer-substring beg end)))
 
 ;;}}}
 
@@ -887,12 +1080,34 @@ indicating the arrival  of new mail when displaying the mode line."
 	   (if prefix "" "locally")))
 
 ;;}}}
-;;{{{  Speak mode line information 
+;;{{{  Speak mode line information
+
+
+;;;compute current line number 
+(defsubst emacspeak-get-current-line-number()
+  (let ((start (point)))
+  (save-excursion
+    (save-restriction
+      (widen)
+    (goto-char (point-min))
+     (+ 1 (count-lines start (point)))))))
+
+;;; make line-number-mode buffer local
+(declaim (special line-number-mode))
+(make-variable-buffer-local 'line-number-mode)
+(setq-default line-number-mode nil)
+
+
+;;; make column-number-mode buffer local
+(declaim (special column-number-mode))
+(make-variable-buffer-local 'column-number-mode)
+(setq-default column-number-mode nil)
 
 (defun emacspeak-speak-mode-line ()
   "Speak the mode-line. "
   (interactive)
   (declare (special  mode-name major-mode
+                     column-number-mode line-number-mode
                      emacspeak-mail-alert))
   (dtk-stop)
   (emacspeak-dtk-sync)
@@ -903,13 +1118,22 @@ indicating the arrival  of new mail when displaying the mode line."
     (when buffer-read-only
       (dtk-tone 250 50))
     (when  emacspeak-mail-alert
-        (and (emacspeak-mail-alert-user)
-             (dtk-tone 450 75)))
+      (and (emacspeak-mail-alert-user)
+           (dtk-tone 450 75)))
     (dtk-speak
-     (format  "%s  %s"
+     (format  "%s %s %s  %d%%  %s"
+              (if line-number-mode
+                  (format "line %d"
+                          (emacspeak-get-current-line-number))
+                "")
+              (if column-number-mode
+                  (format "Column %d"
+                          (current-column))
+                          "")
               (if  (buffer-file-name ) 
                   (file-name-nondirectory  (buffer-file-name ))
                 (buffer-name ))
+              (emacspeak-get-current-percentage-into-buffer)
               (if  major-mode major-mode "")))))
 
 (defun emacspeak-speak-minor-mode-line ()
@@ -917,7 +1141,6 @@ indicating the arrival  of new mail when displaying the mode line."
   (interactive)
   (declare (special minor-mode-alist))
   (force-mode-line-update)
-  (let ((dtk-stop-immediately nil ))
     (dtk-speak
      (format "Active minor modes:  %s" 
              (mapconcat
@@ -930,17 +1153,14 @@ indicating the arrival  of new mail when displaying the mode line."
                                   (eval value)
                                 value))
                              (t "")))))
-              minor-mode-alist " ")))))
-
+              minor-mode-alist " "))))
 
 (defun emacspeak-speak-line-number ()
 "Speak the line number of the current line."
 (interactive)
-(let ((start (point)))
-  (save-excursion
-    (goto-char (point-min))
-    (message "Line %s"
-     (+ 1 (count-lines start (point)))))))
+(message "line %d"
+         (emacspeak-get-current-line-number)))
+
 ;;}}}
 ;;{{{  Speak text without moving point
 
@@ -954,7 +1174,7 @@ offset. Default  is to speak the previous line. "
      ((zerop arg) (emacspeak-speak-line ))
      ((zerop (forward-line arg))
       (emacspeak-speak-line ))
-     (t (dtk-speak "Not that many lines in buffer. ")))))
+     (t (dtk-speak "Not that many lines in buffer ")))))
 
 (defun emacspeak-read-previous-line(&optional arg)
   "Read previous line, specified by an offset, without moving.
@@ -978,7 +1198,7 @@ offset. Default  is to speak the previous word. "
      ((forward-word arg)
       (skip-syntax-forward " ")
       (emacspeak-speak-word 1 ))
-     (t (dtk-speak "Not that many words. ")))))
+     (t (dtk-speak "Not that many words ")))))
 
 (defun emacspeak-read-previous-word(&optional arg)
   "Read previous word, specified as a prefix arg, without moving.
@@ -1035,12 +1255,50 @@ to command yank."
   (dtk-dispatch 
    (read-from-minibuffer"Enter Dectalk command string: ")))
 
+(defun emacspeak-speak-string-to-phone-number (string)
+  "Convert alphanumeric phone number to true phone number"
+  (setq string (downcase string ))
+  (let ((i 0))
+  (loop for character across string
+        do
+        (aset string i 
+        (case character
+          (?a  ?2)
+          (?b ?2)
+          (?c ?2)
+          (?d ?3)
+          (?e ?3)
+          (?f ?3)
+          (?g ?4)
+          (?h ?4)
+          (?i ?4)
+          (?j ?5)
+          (?k ?5)
+          (?l ?5)
+          (?m ?6)
+          (?n ?6)
+          (?o ?6)
+          (?p ?7)
+          (?r ?7)
+          (?s ?7)
+          (?t ?8)
+          (?u ?8)
+          (?v ?8)
+          (?w ?9)
+          (?x ?9)
+          (?y ?9)
+          (?q ?1)
+          (?z ?1)
+          (otherwise character)))
+        (incf i))
+  string))
 
 (defun emacspeak-dial-dtk (number)
   "Prompt for and dial a phone number with the Dectalk."
   (interactive "sEnter phone number to dial:")
   (let ((dtk-stop-immediately nil))
-  (dtk-dispatch (format "[:dial %s]" number))
+  (dtk-dispatch (format "[:dial %s]"
+                        (emacspeak-speak-string-to-phone-number number)))
   (sit-for 4)))
 
 (defun emacspeak-dtk-speak-version ()
@@ -1100,7 +1358,7 @@ achieved by a change in voice personality."
   (let ((key "")
         (position (point ))
         (continue t )
-        (message (format "Press any key to execute %s again" command)))
+        (message (format "Press space to execute %s again" command)))
     (while continue
       (call-interactively command )
       (cond
@@ -1111,10 +1369,10 @@ achieved by a change in voice personality."
                   ;(sit-for 2)
                   (read-key-sequence message )))
           (when(and (stringp key)
-                    (=  7  (string-to-char key )))
+                    (not (=  32  (string-to-char key ))))
             (dtk-stop)
             (setq continue nil )))))
-    (dtk-speak "Exited continuous mode. ")))
+    (dtk-speak "Exited continuous mode ")))
 
 (defun emacspeak-speak-continuously ()
   "Speak a buffer continuously.
@@ -1131,6 +1389,11 @@ Pressing  C-g breaks out, leaving point on last chunk that was spoken.
       (error "You specified an invalid key sequence. " ))
     (emacspeak-execute-repeatedly command)))
 
+(defun emacspeak-speak-browse-buffer ()
+  "Browse the current buffer by reading it a paragraph at a time."
+  (interactive)
+  (emacspeak-execute-repeatedly 'forward-paragraph))
+
 (defvar emacspeak-read-line-by-line-quotient 10
   "Determines behavior of emacspeak-read-line-by-line.")
 
@@ -1173,7 +1436,7 @@ To leave, press \\[keyboard-quit]. "
       (call-interactively 'describe-key-briefly)
       (if (= last-input-event 7) 
           (setq continue nil )))
-      (message "Leaving learn mode. ")))
+      (message "Leaving learn mode ")))
 
 ;;}}}
 ;;{{{ comint
@@ -1294,13 +1557,8 @@ Will be improved if it proves useful."
 (defun emacspeak-speak-current-percentage ()
   "Announce the percentage into the current buffer."
   (interactive)
-  (let* ((pos (point))
-	 (total (buffer-size))
-	 (percent (if (> total 50000)
-		      ;; Avoid overflow from multiplying by 100!
-		      (/ (+ (/ total 200) (1- pos)) (max (/ total 100) 1))
-		    (/ (+ (/ total 2) (* 100 (1- pos))) (max total 1)))))
-    (message "Point is  %d%% into  the current buffer" percent )))
+  (message "Point is  %d%% into  the current buffer"
+           (emacspeak-get-current-percentage-into-buffer )))
 
 ;;}}}
 ;;{{{  Speak the last message again:
@@ -1379,12 +1637,12 @@ Speak the window contents after scrolling. "
         (scroll-other-window  nil)
       (error (setq error t)))
     (if error
-        (message "There is no other window. ")
+        (message "There is no other window ")
       (save-window-excursion
           (other-window 1)
           (cond
            ((= start (window-start) )
-            (message "At bottom of other window. "))
+            (message "At bottom of other window "))
            (t (emacspeak-auditory-icon 'scroll)
               (emacspeak-speak-region (window-start ) (window-end ))))))))
           
@@ -1405,12 +1663,12 @@ Speak the window contents after scrolling. "
         (scroll-other-window  (- height ))
       (error (setq error t )))
       (if error
-          (message "There is no other window. ")
+          (message "There is no other window ")
         (save-window-excursion
           (other-window 1)
           (cond
            ((= start (window-start) )
-            (message "At top of other window. "))
+            (message "At top of other window "))
            (t (emacspeak-auditory-icon 'scroll)
               (emacspeak-speak-region (window-start) (window-end ))))))))
 
@@ -1427,7 +1685,7 @@ Numeric prefix arg can specify number of lines to move. "
           (end-of-line)
           (setq residue (forward-line count))
           (cond
-           ((> residue 0) (message "At bottom of other window. "))
+           ((> residue 0) (message "At bottom of other window "))
            (t (set-window-point (get-buffer-window (current-buffer ))
                                 (point))
             (emacspeak-speak-line ))))
@@ -1446,7 +1704,7 @@ Numeric prefix arg can specify number of lines to move. "
           (end-of-line)
           (setq residue (forward-line (- count)))
           (cond
-           ((> 0 residue) (message "At top of other window. "))
+           ((> 0 residue) (message "At top of other window "))
            (t (set-window-point (get-buffer-window (current-buffer ))
                                 (point))
             (emacspeak-speak-line ))))
@@ -1499,7 +1757,7 @@ Semantics  of `other' is the same as for the builtin emacs command
   (if prompt
       (message 
        (format "Press s to speak start of %s, r for rest of  %s. \
- Any  key for entire %s. "
+ Any  key for entire %s "
                unit-name unit-name unit-name )))
   (let ((char (read-char )))
     (cond
@@ -1630,10 +1888,11 @@ start end personality"
             (read
              (completing-read "Use personality: "
                               personality-table nil t ))))
+    (ems-modify-buffer-safely
     (operate-on-rectangle
      (function (lambda ( start-seg begextra endextra )
                  (emacspeak-put-personality start-seg  (point) personality )))
-     start end  nil)))
+     start end  nil))))
 
 (defun emacspeak-voicify-region (start end &optional personality )
   "Voicify the current region.
@@ -1775,7 +2034,7 @@ Signals end of buffer. "
           (setq skip (skip-syntax-forward " "))
           (cond
            ((zerop skip)
-            (message "Did not move. "))
+            (message "Did not move "))
            ((eobp)
             (message "At end of buffer"))
            (t(emacspeak-auditory-icon 'large-movement )
@@ -1796,7 +2055,7 @@ Signals beginning  of buffer. "
           (setq skip (skip-syntax-backward " "))
           (cond
            ((zerop skip)
-            (message "Did not move. "))
+            (message "Did not move "))
            ((bobp )
             (message "At start  of buffer"))
            (t (beginning-of-line)
@@ -1807,40 +2066,6 @@ Signals beginning  of buffer. "
 
 
 
-;;}}}
-;;{{{  Apply audio annotations
-
-;;; prompt for auditory icon with completion
-
-(defsubst ems-ask-for-auditory-icon ()
-  (declare (special emacspeak-sounds-table))
-  (intern
-   (completing-read  "Select sound:"
-                  (mapcar
-                   (lambda (xx)
-                     (let ((x (car xx)))
-                     (list
-                      (format "%s" x)
-                      (format "%s" x ))))
-                   emacspeak-sounds-table))))
-
-(defun emacspeak-audio-annotate-paragraphs (&optional prefix)
-  "Set property auditory-icon at front of all paragraphs.
-Interactive prefix arg prompts for sound cue to use"
-  (interactive "P")
-    (save-excursion
-      (goto-char (point-max))
-      (ems-modify-buffer-safely
-       (let ((sound-cue  (if prefix
-                             (ems-ask-for-auditory-icon)
-                           'paragraph)))
-          (while (not (bobp))
-            (backward-paragraph)
-            (put-text-property  (point) (1+ (point ))
-                                'auditory-icon sound-cue ))))))
-        
-        
-
 ;;}}}
 ;;{{{  Display properties conveniently
 
@@ -1876,7 +2101,7 @@ If no property is set, show a message and exit. "
       (intern
        (completing-read  "Display property: "
                         emacspeak-property-table ))))
-    (t (message "No property set at point. ")
+    (t (message "No property set at point ")
        nil))))
   (declare (special emacspeak-property-table))
   (if property
@@ -1966,7 +2191,7 @@ Warning! Contents of file filename will be overwritten."
 ;;}}}
 ;;{{{  translate faces to voices
 (defun voice-lock-voiceify-faces ()
-  (interactive)
+  "Map faces to personalities"
   (declare (special voice-lock-mode))
   (save-excursion
     (goto-char (point-min))
@@ -1995,6 +2220,58 @@ Warning! Contents of file filename will be overwritten."
 (dtk-define-voice-alias 'excerpt 'annotation-voice )
 
 
+;;}}}
+;;{{{  completion helpers
+
+;;{{{ switching to completions window from minibuffer:
+
+(defsubst emacspeak-get-minibuffer-contents ()
+  (save-excursion
+    (set-buffer (window-buffer (minibuffer-window)))
+    (buffer-string)))
+
+;;; Make all occurrences of string inaudible 
+(defsubst emacspeak-make-string-inaudible(string)
+(unless (string-match "^ *$" string)
+  (save-excursion
+    (goto-char (point-min))
+    (save-match-data 
+      (ems-modify-buffer-safely
+       (while (search-forward string nil t)
+         (put-text-property (match-beginning 0)
+                            (match-end 0)
+                            'personality 'inaudible)))))))
+
+(defun emacspeak-switch-to-completions-window ()
+  "Jump to the *Completions* buffer if it is active.  We make
+the current minibuffer contents (which is obviously the
+prefix for each entry in the completions buffer) inaudible
+to reduce chatter."
+  (interactive)
+  (declare (special voice-lock-mode))
+  (let ((completions-buffer (get-buffer "*Completions*"))
+        (current-entry (emacspeak-get-minibuffer-contents)))
+    (cond
+     ((and completions-buffer
+           (window-live-p (get-buffer-window completions-buffer )))
+      (select-window  (get-buffer-window completions-buffer ))
+      (when (interactive-p)
+        (unless (get-text-property (point) 'mouse-face)
+          (goto-char (next-single-property-change (point)
+                                                  'mouse-face )))
+        (setq voice-lock-mode t)
+        (when (and  current-entry 
+                    (> (length current-entry) 0))
+          (emacspeak-make-string-inaudible current-entry))
+        (dtk-toggle-splitting-on-white-space)
+        (dtk-speak
+         (emacspeak-get-current-completion-from-completions)))
+      (emacspeak-auditory-icon 'select-object))
+     (t (message "No completions")))))
+
+
+;;}}}
+
 ;;}}}
 (provide 'emacspeak-speak )
 ;;{{{ end of file 
@@ -2003,22 +2280,4 @@ Warning! Contents of file filename will be overwritten."
 ;;; folded-file: t
 ;;; end: 
 
-
-(defun emacspeak-switch-to-completions-window ()
-  "Jump to the *Completions* buffer if it is active. "
-  (interactive)
-  (let ((completions-buffer (get-buffer "*Completions*")))
-    (cond
-     ((and completions-buffer
-           (window-live-p (get-buffer-window completions-buffer )))
-          (select-window  (get-buffer-window completions-buffer ))
-          (when (interactive-p)
-            (unless (get-text-property (point) 'mouse-face)
-              (goto-char (next-single-property-change (point)
-                                                      'mouse-face )))
-               (dtk-speak
-           (emacspeak-get-current-completion-from-completions)))
-          (emacspeak-auditory-icon 'select-object))
-     (t (message "No completions")))))
-
 ;;}}}
