@@ -1,4 +1,4 @@
-;;; $Id: emacspeak-table-ui.el,v 5.0 1996/11/22 18:04:26 raman Exp $
+;;; $Id: emacspeak-table-ui.el,v 7.0 1997/11/13 15:32:30 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description: Emacspeak table handling module
 ;;; Keywords:emacspeak, audio interface to emacs tables are structured
@@ -7,14 +7,14 @@
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com
 ;;; A speech interface to Emacs |
-;;; $date: $ |
-;;;  $Revision: 5.0 $ | 
+;;; $Date: 1997/11/13 15:32:30 $ |
+;;;  $Revision: 7.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-
+;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1995 by T. V. Raman Adobe Systems Incorporated 
 ;;; All Rights Reserved. 
 ;;;
@@ -39,6 +39,7 @@
 (require 'cl)
 (declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-table)
+(require 'emacspeak-tabulate)
 (require 'emacspeak-fix-interactive)
 (require 'emacspeak-speak)
 (require 'emacspeak-sounds)
@@ -66,7 +67,7 @@
   (define-key emacspeak-table-keymap "\t"
     'emacspeak-table-next-column)
   (define-key emacspeak-table-keymap  '[S-tab] 'emacspeak-table-previous-column)
-(define-key emacspeak-table-keymap "g" 'emacspeak-table-goto)
+(define-key emacspeak-table-keymap "j" 'emacspeak-table-goto)
 (define-key emacspeak-table-keymap '[up] 'emacspeak-table-previous-row)
 (define-key emacspeak-table-keymap '[down] 'emacspeak-table-next-row)
 (define-key emacspeak-table-keymap '[left] 'emacspeak-table-previous-column)
@@ -80,8 +81,11 @@
 (define-key emacspeak-table-keymap "a"
   'emacspeak-table-select-automatic-speaking-method)
 (define-key emacspeak-table-keymap "s" 'emacspeak-table-search)
+(define-key emacspeak-table-keymap "f" 'emacspeak-table-speak-row-filtered)
+(define-key emacspeak-table-keymap "g" 'emacspeak-table-speak-column-filtered)
 (define-key emacspeak-table-keymap "h"
   'emacspeak-table-search-headers)
+(define-key emacspeak-table-keymap "k" 'emacspeak-table-copy-to-clipboard)
 (define-key emacspeak-table-keymap "T" 'emacspeak-table-goto-top)
 (define-key emacspeak-table-keymap "B" 'emacspeak-table-goto-bottom)
 (define-key emacspeak-table-keymap "L" 'emacspeak-table-goto-left)
@@ -100,8 +104,12 @@ visual display is kept in sync with the speech you hear; however
 Emacspeak is examining the entire table in order to speak the current
 cell content intelligently.
 
-You can interactively specify that emacspeak should speak either the row or column header (or both) while speaking each cell.
-You can also move to a specific row or column by searching the cell contents or by searching the row or column headers to locate items of interest.
+You can interactively specify that emacspeak should speak either the row or
+column header (or both) while speaking each cell.  You can also specify a row
+or column filter that should be applied when speaking entire rows or columns
+--this lets you view slices of a table.  You can move to a specific row or
+column by searching the cell contents or by searching the row or column
+headers to locate items of interest.
 
 Here is a short description of the special commands provided in this mode.
 
@@ -119,9 +127,10 @@ When searching headers remember that row 0 is the column header, and that column
 h               emacspeak-table-search-headers
 s               emacspeak-table-search
 
-The next command lets you specify how cell contents should be spoken.
-Specify one of: `b' for both, `c' for column, or `r' for row --table
-cells with then be spoken with both (or either)row and column headers,.
+The next command lets you specify how cell contents should be spoken.  Specify
+one of: `b' for both, `c' for column, `r' for row, `f' for row filtering and
+`g' for column filtering. --table cells with then be spoken with both (or
+either)row and column headers, or with the filter applied.
 
 a               emacspeak-table-select-automatic-speaking-method
 
@@ -139,24 +148,31 @@ right               emacspeak-table-next-column
 left               emacspeak-table-previous-column
 down               emacspeak-table-next-row
 up               emacspeak-table-previous-row
-g               emacspeak-table-goto
+j               emacspeak-table-goto
 S-tab               emacspeak-table-previous-column
-TAB               emacspeak-table-next-column"
-  (declare (special emacspeak-table-keymap
-                    voice-lock-mode))
-  (use-local-map emacspeak-table-keymap)
-  (setq voice-lock-mode t)
-  (setq major-mode 'emacspeak-table-mode)
-  (setq mode-name "table")
-  (put-text-property (point-min) (point-max)
-                     'point-entered 'emacspeak-table-point-motion-hook)
-  (not-modified)
-  (setq buffer-read-only t)
-  (emacspeak-auditory-icon 'alarm)
-  (emacspeak-speak-mode-line))
+TAB               emacspeak-table-next-column
+
+Row and Column Filtering
+
+Filtering is designed to let you view slices of a table.
+They are specified as lists of numbers and strings.
+The concept is best explained with an example.
+
+A row filter specifies which of the entries in the current row should be
+spoken.Entries are numbered starting with 0.  Thus, when working with a table
+having 8 columns, a row filter of ( 1 2 3) will speak only entries 1 2 and 3.
+Use the sample table matrix.dat to familiarize yourself with this
+feature. Note that you can intersperse meaningful strings in the list that
+specifies the filter"
+(declare (special emacspeak-table-keymap)) (use-local-map
+emacspeak-table-keymap) (set (make-local-variable 'voice-lock-mode) t) (setq
+major-mode 'emacspeak-table-mode) (setq mode-name "table") (put-text-property
+(point-min) (point-max) 'point-entered 'emacspeak-table-point-motion-hook)
+(not-modified) (setq buffer-read-only t) (emacspeak-auditory-icon 'alarm)
+(emacspeak-speak-mode-line))
 
 ;;}}}
-;;{{{  current position
+;;{{{  speaking current entry 
 
 (defsubst emacspeak-table-synchronize-display ()
   "Bring visual display in sync with internal representation"
@@ -250,6 +266,107 @@ TAB               emacspeak-table-next-column"
                           (emacspeak-table-current-element
                            emacspeak-table)))))))
 
+(defsubst emacspeak-table-get-entry-with-headers  (row column &optional row-head-p col-head-p)
+  "Return   both row and column header and table element"
+  (interactive)
+  (declare (special emacspeak-table
+                    emacspeak-table-column-header-personality
+                    emacspeak-table-row-header-personality))
+  (and (boundp 'emacspeak-table)
+       (let ((column-head (format "%s"
+                                  (emacspeak-table-column-header-element emacspeak-table column)))
+             (row-head (format "%s"
+                               (emacspeak-table-row-header-element
+                                emacspeak-table row))))
+         (and row-head-p 
+              (put-text-property 0 (length row-head)
+                                 'personality
+                                 emacspeak-table-row-header-personality
+                                 row-head))
+         (and col-head-p
+              (put-text-property 0 (length column-head)
+                                 'personality
+                                 emacspeak-table-column-header-personality column-head))
+         (concat
+          (if row-head-p
+              row-head
+            "")
+          " "
+          (if col-head-p column-head  "")
+          (format " %s"
+                  (emacspeak-table-this-element emacspeak-table row column))))))
+
+(defvar emacspeak-table-speak-row-filter nil
+  "Template specifying how a row is filtered before it is spoken.")
+
+(make-variable-buffer-local 'emacspeak-table-speak-row-filter)
+
+(defun emacspeak-table-speak-row-filtered  (&optional prefix)
+  "Speaks a table row after applying a specified row filter.
+Optional prefix arg prompts for a new filter."
+  (interactive "P")
+  (declare (special emacspeak-table-speak-row-filter
+                    emacspeak-table))
+  (unless (and  emacspeak-table-speak-row-filter
+                    (listp emacspeak-table-speak-row-filter)
+                    (not prefix))
+    (setq emacspeak-table-speak-row-filter
+          (read-minibuffer "Specify row filter as a list: " "(")))
+  (dtk-speak
+   (mapconcat
+    (function
+     (lambda (token)
+       (cond
+        ((stringp token) token)
+        ((numberp token)
+         (emacspeak-table-get-entry-with-headers
+          (emacspeak-table-current-row emacspeak-table) token))
+        ((and (listp token)
+              (numberp (first token))
+              (numberp (second token )))
+         (emacspeak-table-get-entry-with-headers
+          (first token)
+          (second token)))
+        (t  (format "%s" token)))))
+    emacspeak-table-speak-row-filter 
+    " ")))
+
+(defvar emacspeak-table-speak-column-filter nil
+  "Template specifying how a column is filtered before it is spoken.")
+
+(make-variable-buffer-local 'emacspeak-table-speak-column-filter)
+
+(defun emacspeak-table-speak-column-filtered  (&optional prefix)
+  "Speaks a table column after applying a specified column filter.
+Optional prefix arg prompts for a new filter."
+  (interactive "P")
+  (declare (special emacspeak-table-speak-column-filter
+                    emacspeak-table))
+  (unless (and  emacspeak-table-speak-column-filter
+                 (listp emacspeak-table-speak-column-filter)
+                 (not prefix))
+    (setq emacspeak-table-speak-column-filter
+          (read-minibuffer "Specify column filter as a list: " "(")))
+  (dtk-speak
+   (mapconcat
+    (function
+     (lambda (token)
+       (cond
+        ((stringp token) token)
+        ((numberp token)
+         (emacspeak-table-get-entry-with-headers
+           token
+           (emacspeak-table-current-column emacspeak-table)))
+        ((and (listp token)
+              (numberp (first token))
+              (numberp (second token )))
+         (emacspeak-table-get-entry-with-headers
+          (first token)
+          (second token)))
+        (t  (format "%s" token)))))
+    emacspeak-table-speak-column-filter 
+    " ")))
+
 ;;}}}
 ;;{{{  what to do when point moves
 
@@ -275,7 +392,6 @@ emacspeak distribution contains some sample tables --these are the
 CalTrain schedules.  Execute command `describe-mode' bound to
 \\[describe-mode] in a buffer that is in emacspeak table mode to read
 the documentation on the table browser."
-
   (interactive "FEnter filename containing table data: ")
   (declare (special positions))
   (let ((buffer (get-buffer-create
@@ -328,11 +444,72 @@ the documentation on the table browser."
 
 (emacspeak-fix-interactive-command-if-necessary 'emacspeak-table-find-file)
 
+;;}}}
+;;{{{ Processing a region of tabular data
+(defun emacspeak-table-display-table-in-region (start end)
+  "Recognize tabular data in current region and display it in table
+browsing mode in a a separate buffer.
+emacspeak table mode is designed to let you browse tabular data using
+all the power of the two-dimensional spatial layout while giving you
+sufficient contextual information.  The tables subdirectory of the
+emacspeak distribution contains some sample tables --these are the
+CalTrain schedules.  Execute command `describe-mode' bound to
+\\[describe-mode] in a buffer that is in emacspeak table mode to read
+the documentation on the table browser."
+  (interactive "r")
+  (declare (special positions))
+  (let ((buffer (get-buffer-create
+                 (format  "*table-%s*"
+                          (or (buffer-name)
+                              "scratch"))))
+        (table nil)
+        (data nil)
+        (i 0)
+        (j 0)
+        (count 0)
+        (row-start 1)
+        (column-start 1))
+    (setq table (make-emacspeak-table
+                 (ems-tabulate-parse-region start end)))
+    (save-excursion
+      (set-buffer buffer)
+      (let ((inhibit-read-only t))
+        (erase-buffer)
+        (set (make-local-variable 'emacspeak-table) table)
+        (set (make-local-variable 'positions) (make-hash-table))
+        (setq count (1-  (emacspeak-table-num-columns table)))
+        (loop for row across (emacspeak-table-elements table)
+              do
+              (loop for element across row
+                    do 
+                    (setf
+                     (gethash
+                      (intern (format "element:%s:%s" i j ))
+                      positions)
+                     (point))
+                    (insert
+                     (format "%s%s"
+                             (emacspeak-table-this-element table i j )
+                             (if (=  j count)
+                                 "\n"
+                               "\t")))
+                    (put-text-property column-start (point)
+                                       'column j)
+                    (setq column-start (point))
+                    (incf j))
+              (setq j 0)
+              (put-text-property row-start (point) 'row i)
+              (setq row-start (point))
+              (incf i))
+        (emacspeak-table-mode)
+        (goto-char (point-min))))
+    (switch-to-buffer buffer)))
+
 ;;}}}
 ;;{{{ select default speaking action
 
 (defvar emacspeak-table-select-automatic-speaking-method-prompt
-  "Select: b both c column d default r row "
+  "Select: b both c column d default r row f filter row g filter column "
   "Prompt to display when selecting automatic speaking method for
 table elements")
 
@@ -349,6 +526,8 @@ browsing table elements"
             (?c 'emacspeak-table-speak-column-header-and-element)
             (?r 'emacspeak-table-speak-row-header-and-element)
             (?d 'emacspeak-table-speak-current-element)
+            (?f 'emacspeak-table-speak-row-filtered)
+            (?g 'emacspeak-table-speak-column-filtered)
             (?. 'emacspeak-table-speak-coordinates)
             (otherwise (message "Invalid method specified")
                        emacspeak-table-speak-element)))
@@ -558,7 +737,155 @@ match, makes the matching row or column current."
     (funcall emacspeak-table-speak-element)))
 
 ;;}}}
+;;{{{ cutting and pasting tables:
 
+;;; Implementing table editing and table clipboard.
+;;{{{ variables 
+
+(defvar emacspeak-table-clipboard nil
+  "Variable to hold table copied to the clipboard.")
+
+;;}}}
+;;{{{  define table markup structure and accessors 
+
+(defstruct emacspeak-table-markup
+  table-start
+  table-end
+  row-start
+  row-end
+  col-start
+  col-end
+  col-separator )
+
+(defvar emacspeak-table-markup-table  (make-hash-table)
+  "Hash table to hold mapping between major modes and mode specific
+table markup.")
+
+(defsubst emacspeak-table-markup-set-table (mode markup)
+(declare (special emacspeak-table-markup-table))
+(setf  (gethash mode emacspeak-table-markup-table) markup ))
+
+(defsubst emacspeak-table-markup-get-table (mode )
+(declare (special emacspeak-table-markup-table))
+  (or (cl-gethash mode emacspeak-table-markup-table)
+      (cl-gethash emacspeak-table-markup-table 'text-mode)))
+
+ ;;}}}
+;;{{{  define table markup for the various modes of interest
+
+(emacspeak-table-markup-set-table 'html-helper-mode
+                                  (make-emacspeak-table-markup
+                                   :table-start "<TABLE>\n"
+                                   :table-end "</TABLE>\n"
+                                   :row-start "<TR>\n"
+                                   :row-end "</TR>\n"
+                                   :col-start "<TD>\n"
+                                   :col-end "</TD>\n"
+                                   :col-separator ""))
+
+
+(emacspeak-table-markup-set-table 'latex2e-mode
+                                  (make-emacspeak-table-markup
+                                   :table-start "\\begin{tabular}\n"
+                                   :table-end "\\end{tabular}\n"
+                                   :row-start ""
+                                   :row-end "\\\\\n"
+                                   :col-start ""
+                                   :col-end ""
+                                   :col-separator " & "))
+
+(emacspeak-table-markup-set-table 'latex-mode
+                                  (emacspeak-table-markup-get-table
+                                   'latex2e-mode))
+
+(emacspeak-table-markup-set-table 'LaTeX-mode
+                                  (emacspeak-table-markup-get-table
+                                   'latex2e-mode))
+
+(emacspeak-table-markup-set-table 'TeX-mode
+                                  (emacspeak-table-markup-get-table
+                                   'latex2e-mode))
+
+(emacspeak-table-markup-set-table 'text-mode
+                                  (make-emacspeak-table-markup
+                                   :table-start
+                                   "\n------------------------------------------------------------\n"
+                                   :table-end
+                                   "\n------------------------------------------------------------\n"
+                                   :row-start ""
+                                   :row-end "\n"
+                                   :col-start ""
+                                   :col-end ""
+                                   :col-separator "\t"))
+
+(emacspeak-table-markup-set-table 'fundamental-mode
+                                  (emacspeak-table-markup-get-table
+                                   'text-mode))
+
+;;}}}
+;;{{{ copy and paste tables 
+
+(defun emacspeak-table-copy-to-clipboard ()
+  "Copy table in current buffer to the table clipboard.
+Current buffer must be in emacspeak-table mode."
+  (interactive)
+  (declare (special emacspeak-table-clipboard
+                    emacspeak-table))
+  (unless (eq major-mode 'emacspeak-table-mode)
+    (error
+     "emacspeak-table-copy-to-clipboard can be used only in
+emacspeak-table-mode. "))
+  (cond
+   ((boundp 'emacspeak-table)
+  (setq emacspeak-table-clipboard emacspeak-table)
+  (message "Copied current table to emacspeak table clipboard."))
+  (t (error "Cannot find table in current buffer"))))
+
+
+(defun emacspeak-table-paste-from-clipboard ()
+  "Paste the emacspeak table clipboard into the current buffer.
+Use the major  mode of this buffer to  decide what kind of table
+markup to use."
+  (interactive)
+  (declare (special emacspeak-table-clipboard
+                    emacspeak-table))
+  (let ((mode  major-mode)
+        (markup nil)
+        (table (emacspeak-table-elements emacspeak-table-clipboard))
+        (read-only buffer-read-only)
+        (table-start nil)
+        (table-end nil)
+        (row-start nil)
+        (row-end nil)
+        (col-start nil)
+        (col-end nil)
+        (col-separator nil))
+    (cond
+     (read-only
+      (error "Cannot paste into read only buffer."))
+     (t (setq markup  (emacspeak-table-markup-get-table mode))
+        (setq table-start (emacspeak-table-markup-table-start markup)
+              table-end (emacspeak-table-markup-table-end markup)
+              row-start (emacspeak-table-markup-row-start markup)
+              row-end (emacspeak-table-markup-row-end markup)
+              col-start (emacspeak-table-markup-col-start markup)
+              col-end (emacspeak-table-markup-col-end markup)
+              col-separator (emacspeak-table-markup-col-separator markup))
+        (insert (format "%s" table-start))
+        (loop for row across table
+              do
+              (insert (format "%s" row-start))
+              (loop for column across row
+                    do
+                    (insert (format "%s %s %s"
+                                    col-start column col-end  ))
+                    (insert (format "%s" col-separator)))
+              (insert (format "%s" row-end)))
+        (insert (format "%s" table-end))))))
+
+ ;;}}}
+
+;;}}}
 (provide  'emacspeak-table-ui)
 ;;{{{  emacs local variables 
 
