@@ -1,19 +1,20 @@
-;;; $Id: emacspeak.el,v 5.0 1996/11/22 18:04:26 raman Exp $
+;;; $Id: emacspeak.el,v 7.0 1997/11/13 15:32:30 raman Exp $
 ;;; $Author: raman $ 
 ;;; Description:  Emacspeak: A speech interface to Emacs
 ;;; Keywords: Emacspeak, Speech, Dectalk,
 ;;{{{  LCD Archive entry: 
+
 ;;; LCD Archive Entry:
 ;;; emacspeak| T. V. Raman |raman@adobe.com 
 ;;; A speech interface to Emacs |
-;;; $date: $ |
-;;;  $Revision: 5.0 $ | 
+;;; $Date: 1997/11/13 15:32:30 $ |
+;;;  $Revision: 7.0 $ | 
 ;;; Location undetermined
 ;;;
 
 ;;}}}
 ;;{{{  Copyright:
-
+;;;Copyright (C) 1995, 1996, 1997  T. V. Raman  Adobe Systems Incorporated
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -53,7 +54,7 @@
 ;;{{{  Setting up things: 
 
 (defconst emacspeak-version
-  (let ((x "$Revision: 5.0 $"))
+  (let ((x "$Revision: 7.0 $"))
     (string-match "[0-9.]+" x)
     (substring x (match-beginning 0)
                (match-end 0)))
@@ -64,8 +65,32 @@
 
 ;;}}}
 ;;{{{ add autoloads and set some load hooks
+(autoload 'emacspeak-remote-startup-local-server
+   "emacspeak-remote"
+"Startup  speech server.
+Use this when you are running emacspeak on a remote machine and want
+to have speech output on the local desktop.
+Unlike the simpler rsh based remote-tcl solution
+(see file remote-tcl in the emacspeak distribution)
+this facility allows you
+to get remote speech feedback in cases where the remote machine cannot
+use rsh on your local desktop."
+  t)
 
-(autoload 'voice-lock-mode "voice-lock" t "voice lock mode")
+(autoload 'emacspeak-remote-connect-to-server "emacspeak-remote"
+"Connect to a remote speech server.
+Use this when you are running emacspeak on a remote machine and want
+to have speech output on the local desktop.
+Unlike the simpler rsh based remote-tcl solution
+(see file remote-tcl in the emacspeak distribution)
+this command emacspeak-remote-connect-to-server allows you
+to get remote speech feedback in cases where the remote machine cannot
+use rsh on your local desktop."
+  t)
+
+(autoload 'emacspeak-eterm-remote-term "emacspeak-eterm"
+   "Create a terminal to rlogin into a remote host" t)
+(autoload 'voice-lock-mode "voice-lock" "voice lock mode" t)
     (autoload 'emacspeak-toggle-auditory-icons "emacspeak-sounds"
       "Ask emacspeak to use auditory icons" t)
     (autoload 'emacspeak-tabulate-region "emacspeak-tabulate" 
@@ -80,42 +105,122 @@
 ;;{{{ Emacspeak:
 
 (defvar emacspeak-emacs-commands-to-fix 
-  '(locate-library
-    insert-file
-    string-rectangle 
-    point-to-register
-    find-file-other-window
-    switch-to-buffer-other-window
-    jump-to-register
-    describe-key
-    find-file-read-only-other-window
-    dtk-set-rate
-    cd
-    goto-line
-    load-file
-    window-configuration-to-register
-    load-library
-    view-register
-    prepend-to-register
-    describe-key-briefly
-    append-to-register
-    byte-recompile-directory
-    copy-to-register
-    command-apropos
-    insert-register
-    append-to-file
-    read-abbrev-file
-    copy-rectangle-to-register
-    make-obsolete
-    zap-to-char
-    prepend-to-buffer
-    dtk-set-character-scale
-    set-visited-file-name
+  '(
     ad-compile-function
-    find-file
-    find-file-read-only
+    add-completions-from-file
+    append-to-file
+    append-to-register
+    byte-force-recompile
+    byte-recompile-directory
+    cd
+    comint-run
+    command-apropos
+    copy-rectangle-to-register
     copy-to-buffer
-    emacspeak-dial-dtk)
+    copy-to-register
+    customize-apropos
+    customize-apropos-faces
+    customize-apropos-groups
+    customize-apropos-options
+    define-mail-abbrev
+    define-mail-alias
+    define-mail-alias
+    describe-key
+    describe-key-briefly
+    desktop-save
+    dmacro-load
+    dtk-set-character-scale
+    dtk-set-rate
+    emacspeak-dial-dtk
+    find-file
+    find-file-literally
+    find-file-literally
+    find-file-other-frame
+    find-file-other-window
+    find-file-read-only
+    find-file-read-only-other-frame
+    find-file-read-only-other-window
+    fold-goto-line
+    global-unset-key
+    goto-line
+    insert-file
+    insert-file-literally
+    insert-file-literally
+    insert-register
+    insert-variable
+    jump-to-register
+    list-text-properties-at
+    load-file
+    load-library
+    local-unset-key
+    locate-library
+    mail-attach-file
+    mail-attach-file
+    mail-fcc
+    mail-fcc
+    mail-fcc
+    make-frame-on-display
+    make-frame-on-display
+    make-obsolete
+    makefile-insert-macro
+    makefile-insert-macro
+    makefile-insert-macro
+    makefile-insert-target
+    makefile-insert-target
+    makefile-insert-target
+    message-resend
+    mime-include-audio
+    mime-include-external-anonftp
+    mime-include-external-ftp
+    mime-include-gif
+    mime-include-jpeg
+    mime-include-postscript
+    mime-include-raw-binary
+    mime-include-raw-nonbinary
+    nonincremental-re-search-backward
+    nonincremental-re-search-forward
+    nonincremental-search-backward
+    nonincremental-search-forward
+    point-to-register
+    prefer-coding-system
+    prefer-coding-system
+    prepend-to-buffer
+    prepend-to-register
+    read-abbrev-file
+    recover-file
+    run-at-time
+    run-at-time
+    run-at-time
+    run-with-timer
+    run-with-timer
+    set-buffer-file-coding-system
+    set-buffer-file-coding-system
+    set-buffer-process-coding-system
+    set-buffer-process-coding-system
+    set-foreground-color
+    set-foreground-color
+    set-frame-font
+    set-frame-font
+    set-left-margin
+    set-left-margin
+    set-right-margin
+    set-right-margin
+    set-visited-file-name
+    sort-regexp-fields
+    string-rectangle
+    string-rectangle 
+    switch-to-buffer-other-frame
+    switch-to-buffer-other-window
+    vc-comment-search-forward
+    vc-comment-search-reverse
+    vc-directory
+    vc-rename-file
+    vc-version-diff
+    vc-version-other-window
+    view-register
+    window-configuration-to-register
+    zap-to-char
+    )
   "Precomputed list of interactive functions that have to be fixed.
 Precomputing this saves time at start-up.")
 
@@ -150,7 +255,7 @@ See the online documentation for individual commands and functions for
   
   (interactive)
   (declare (special dtk-program dtk-drivers-alist dtk-driver-prefix
- emacspeak-emacs-commands-to-fix))
+                    emacspeak-emacs-commands-to-fix))
   (let ((driver-flag nil ))
     (load-library "dtk-speak")
     (unless  dtk-program
@@ -173,10 +278,11 @@ time!"))
     (load-library "emacspeak-advice")
     (load-library "emacspeak-replace")
     (mapcar 'emacspeak-fix-interactive-command-if-necessary 
-emacspeak-emacs-commands-to-fix)
+            emacspeak-emacs-commands-to-fix)
     (run-hooks 'emacspeak-startup-hook)
     (emacspeak-dtk-sync)
-    (message  (format "  Press %s to get an   overview of emacspeak  %s. \
+    (emacspeak-export-environment)
+    (message  (format "  Press %s to get an   overview of emacspeak  %s \
  I am  completely operational,  and all my \
 circuits are functioning perfectly! "
                       (substitute-command-keys
@@ -198,10 +304,118 @@ circuits are functioning perfectly! "
 .tab files in the tables subdirectory of emacspeak.
 Describe function of emacspeak-table-find-file will then give you
 detailed documentation on the table browser" t )
+(autoload 'emacspeak-table-display-table-in-region "emacspeak-table-ui"
+"Parse contents of region as tabular data and display it in table
+browsing mode.  Beginners: please do C-e C-t and specify one of the
+.tab files in the tables subdirectory of emacspeak.  Describe function
+of emacspeak-table-find-file will then give you detailed documentation
+on the table browser"
+ t )
+(autoload 'emacspeak-tapestry-describe-tapestry  "emacspeak-tapestry"
+  "Describe layout of buffers in current frame.
+This needs the tapestry.el package used by many Emacs systems like the
+vm mail reader." t)
+
+(autoload 'emacspeak-hide-or-expose-block "emacspeak-hide"
+  "Hide or expose blocks of text that share a common prefix.
+Useful in reading email, block comments in program source etc." t)
+(autoload 'emacspeak-hide-or-expose-all-blocks "emacspeak-hide"
+  "Hide or expose blocks of text that share a common prefix.
+Useful in reading email, block comments in program source etc." t)
+(autoload 'emacspeak-hide-speak-block-sans-prefix "emacspeak-hide"
+  "Speak a block of text sans the prefix that appears on every line of
+the block. Use in conjunction with emacspeak-hide-or-expose-block"
+   t)
+
 
 ;;}}}
 ;;{{{  Advice for Emacs subsystems:
+;;{{{  ange-ftp
+
+(add-hook 'after-load-alist
+          '("ange-ftp"
+            (loop for f in 
+                  '(ange-ftp-copy-file
+                    ange-ftp-delete-file
+                    ange-ftp-kill-ftp-process
+                    ange-ftp-nslookup-host
+                    ange-ftp-rename-file
+                    ange-ftp-set-user)
+                  do
+                  (emacspeak-fix-interactive-command-if-necessary f))))
+
+;;}}}
+;;{{{ imenu 
+
+(add-hook 'after-load-alist
+          '("imenu"
+                        (load "emacspeak-imenu")))
+
+;;}}}
+;;{{{  tempo
+(add-hook 'after-load-alist
+          '("tempo"
+                        (load "emacspeak-tempo")))
+
+;;}}}
+;;{{{  solitaire
+
+(add-hook 'after-load-alist
+          '("solitaire"
+                        (load "emacspeak-solitaire")))
+
+;;}}}
+;;{{{  tetris
+
+(add-hook 'after-load-alist
+          '("tetris"
+                        (load "emacspeak-tetris")))
+
+(defadvice tetris (before emacspeak pre act comp)
+  "Speech enable the game"
+  (require 'emacspeak-tetris))
+
+;;}}}
+;;{{{  bookmark
+
+(add-hook 'after-load-alist
+          '("bookmark"
+                        (load "emacspeak-bookmark")))
+
+;;}}}
+;;{{{  buff-menu
+
+(add-hook 'after-load-alist
+          '("buff-menu"
+                        (load "emacspeak-buff-menu")))
+(defadvice list-buffers (before first pre act comp)
+  "Make sure emacspeak advice is loaded"
+  (require 'emacspeak-buff-menu))
+
+;;}}}
+;;{{{  tar
+
+(add-hook 'after-load-alist
+          '("tar-mode"
+                        (load "emacspeak-tar")))
+
+;;}}}
+;;{{{  arc
+
+(add-hook 'after-load-alist
+          '("arc-mode"
+                        (load "emacspeak-arc")))
+
+;;}}}
+;;{{{  gomoku
+
+(add-hook 'after-load-alist
+          '("gomoku"
+                        (load "emacspeak-gomoku.elc")))
+
+;;}}}
 ;;{{{ rmail
+
 (add-hook 'after-load-alist
           '("rmail"
                         (load "emacspeak-rmail")))
@@ -213,6 +427,42 @@ detailed documentation on the table browser" t )
           '("view"
                         (load "emacspeak-view")))
 
+;;}}}
+;;{{{ enriched text
+
+(add-hook 'after-load-alist
+          '("enriched"
+                        (load "emacspeak-enriched")))
+
+;;}}}
+;;{{{ facemenu 
+
+(add-hook 'after-load-alist
+          '("facemenu"
+                        (require  'emacspeak-facemenu)))
+
+(add-hook 'after-load-alist
+          '("tmm"
+                        (require  'emacspeak-facemenu)))
+
+;;}}}
+;;{{{ find-dired
+
+(add-hook 'after-load-alist
+          '("find-dired"
+                        (mapcar
+                         'emacspeak-fix-interactive-command-if-necessary
+                         (list 'find-dired
+                               'find-name-dired
+                               'find-grep-dired))))
+
+;;}}}
+;;{{{ sgml mode
+
+(add-hook 'after-load-alist
+          '("sgml-mode"
+                        (load "emacspeak-sgml-mode")))
+
 ;;}}}
 ;;{{{  message mode
 
@@ -252,24 +502,14 @@ detailed documentation on the table browser" t )
 ;;{{{ widget
 
 (add-hook 'after-load-alist
-          '("widget-edit"
+          '("wid-edit"
                         (load "emacspeak-widget")))
 ;;}}}
 ;;{{{ w3
 
 (add-hook 'after-load-alist
           '("w3"
-            (progn
-              (declare (special w3-echo-link url-show-status))
-              (condition-case nil
-                  (progn (load-library "w3-speak")
-                         (add-hook 'w3-mode-hook 'w3-speak-mode-hook)
-                         (setq w3-echo-link 'text)
-                         (setq url-show-status nil)
-                         (provide 'emacspeak-w3))
-                (error (emacspeak-auditory-icon 'warn-user)
-                       (load-library "emacspeak-w3")))
-              (emacspeak-fix-interactive-command-if-necessary 'w3-open-local ))))
+            (require 'emacspeak-w3)))
             
 
 ;;}}}
@@ -344,10 +584,10 @@ detailed documentation on the table browser" t )
 
 ;;}}}
 ;;{{{  emacspeak-compile
+(add-hook 'after-load-alist
+          '("compile"
+                        (load "emacspeak-compile")))
 
-(defadvice compile (before emacspeak-load activate)
-  "Load emacspeak-compile if not already loaded."
-  (require 'emacspeak-compile))
 
 ;;}}}
 ;;{{{  calc:
@@ -436,7 +676,7 @@ detailed documentation on the table browser" t )
   "Provide auditory feedback."
   (when (interactive-p)
     (emacspeak-fix-interactive-command-if-necessary 'ielm-change-working-buffer)
-    (message "Welcome to the inferior elisp buffer.")))
+    (message "Welcome to the inferior elisp buffer")))
 
 ;;}}}
 
@@ -517,7 +757,7 @@ detailed documentation on the table browser" t )
   (let ((emacspeak-speak-messages nil))
     ad-do-it)
   (emacspeak-auditory-icon 'news)
-  (message "Gnus is ready. "))
+  (message "Gnus is ready "))
 
 ;;}}}
 ;;{{{ Entertainment --mpuz
@@ -584,7 +824,20 @@ detailed documentation on the table browser" t )
                                    "Description of Problem:"))))
 
 ;;}}}
+;;{{{ exporting emacspeak environment to subprocesses
 
+(defun emacspeak-export-environment ()
+  "This exports emacspeak's system variables to the environment
+so it can be passed to subprocesses."
+  (declare (special emacspeak-dir
+                    emacspeak-play-program
+                    emacspeak-sounds-directory))
+(setenv "EMACSPEAK_DIR" emacspeak-dir)
+   (setenv "EMACSPEAK_SOUNDS_DIR" emacspeak-sounds-directory)
+ (setenv "EMACSPEAK_PLAY_PROGRAM" emacspeak-play-program)
+ )
+
+;;}}}
 (provide 'emacspeak)
 ;;{{{ end of file 
 
